# Balazar
# Copyright (C) 2003 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import soya
from soya import Point, Vector

import balazar.base as base

class Building(soya.World, base.Entity):
  def Goto(self, character):
    from balazar.controler import Goto
    yield Goto(character, self)
    
    
class Market(Building, base.Terraformer):
  radius           = 10.0
  terraform_radius = 10.0
  def __init__(self, parent = None):
    Building.__init__(self, parent)
    base.DecorativeEntity(self, soya.Shape.get("marchee_etalage"))
    
class Farm(Building, base.PhotoScoreable, base.Terraformer):
  radius           = 5.5
  terraform_radius = 5.5
  def __init__(self, parent = None):
    Building.__init__(self, parent, soya.Shape.get("ferme"))
    #Building.__init__(self, parent, soya.Shape.get("scn-village-house"))
    #Building.__init__(self, parent, soya.Shape.get("scn-high-house"))
    
  def get_photo_score(self, photo, camera):
    return base.SHAPES_SCORES.get(self.shape.filename) or base.DEFAULT_SCORE
  
  
class Tower(Building, base.PhotoScoreable, base.Terraformer):
  radius           = 4.0
  terraform_radius = 3.0
  def __init__(self, parent = None):
    Building.__init__(self, parent, soya.Shape.get("tour"))
    
  def get_photo_score(self, photo, camera):
    return base.SHAPES_SCORES.get(self.shape.filename) or base.DEFAULT_SCORE
  


class TrueTemple(Building, base.PhotoScoreable, base.Terraformer):
  radius           = 13.0
  terraform_radius = 15.0
  def __init__(self, parent = None):
    Building.__init__(self, parent)
    self.shape     = soya.Shape.get("marchee")
    self.activated = 0
    
    m = soya.Material()
    m.texture = soya.PARTICLE_DEFAULT_MATERIAL.texture
    
    self.aura = soya.Smoke(self, m, 50)
    self.aura.y = 11.0
    self.aura.scale(4.0, 4.0, 4.0)
    self.aura.set_colors((0.0, 0.0, 0.0, 0.1), (0.0, 0.0, 0.0, 0.8), (0.0, 0.0, 0.0, 0.8), (0.0, 0.0, 0.0, 0.1))
    self.aura.set_sizes ((5.0, 5.0), (20.0, 20.0))
    self.aura.auto_generate_particle = 1

  def set_activated(self, activated):
    if self.activated != activated:
      self.activated = activated
      if activated:
        self.aura.material = soya.Material.get("x_brume_1")
        color = (0.04, 0.08, 0.15, 1.0)
        self.aura.set_colors((0.02, 0.02, 0.02, 1.0), color, color, color, color, (0.02, 0.02, 0.02, 1.0))
      else:
        m = soya.Material()
        m.texture = soya.PARTICLE_DEFAULT_MATERIAL.texture
        self.aura.material = m
        self.aura.set_colors((0.0, 0.0, 0.0, 0.1), (0.0, 0.0, 0.0, 0.8), (0.0, 0.0, 0.0, 0.8), (0.0, 0.0, 0.0, 0.1))
        
        
  def get_photo_score(self, photo, camera):
    return base.SHAPES_SCORES.get(self.shape.filename) or base.DEFAULT_SCORE
  
  
class MorkulTemple(Building, base.Terraformer):
  radius           = 10.0
  terraform_radius = 3.0
  def __init__(self, parent = None):
    Building.__init__(self, parent)
    base.DecorativeEntity(self, soya.Shape.get("temple_morkul"))

