# Balazar
# Copyright (C) 2003-2004 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, math, time, types
import soya, soya.sdlconst as soyasdlconst,  soya.widget as widget
from soya import Point, Vector

from balazar.character import *
from balazar.character import _P, _V
from balazar.item import Item, EquipableItem, set_items_solid, Photo

import balazar.base  as base
import balazar.sound as sound

def Wait(self, duration):
  for i in range(duration): yield State(self)

def Wander(self, duration):
  p = soya.Point(self, (self.level.random.random() - 0.5) * 10.0, 0.0, (self.level.random.random() - 0.75) * 12.0)
  p.convert_to(self.level)
  yield GotoXZ(self, p, 2.0)
  
    
def Goto(self, target, limit = 1.0):
  while self.distance_to(target) > limit:
    tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
    
    if tz < 0.0:
      txz = tx / tz
      
      if   txz < -1.0: yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      elif txz >  1.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      elif self.on_ground and (ty > 3.0) and (tz > -10.0):
        if tz > -2.0: # Reculer pour mieux sauter...
          for i in range(13):
            tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
            txz = tx / tz
            if   txz < -0.3: yield State(self, 0.0, 0.0, 0.2 * self.move_speed, -4.0, "recule", auto_jump = ty > -1.0)
            elif txz >  0.3: yield State(self, 0.0, 0.0, 0.2 * self.move_speed,  4.0, "recule", auto_jump = ty > -1.0)
            else:            yield State(self, 0.0, 0.0, 0.2 * self.move_speed,  0.0, "recule", auto_jump = ty > -1.0)
        yield State(self, 0.0, 0.5, 0.0, 0.0, auto_jump = 1)
      elif txz < -0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed, -4.0, "marche", auto_jump = ty > -1.0)
      elif txz >  0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  4.0, "marche", auto_jump = ty > -1.0)
      else:            yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  0.0, "marche", auto_jump = ty > -1.0)
      
    else:
      if tx < 0.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      else:        yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")

def GotoXZ(self, target, limit = 1.0):
  target = target % self.level
  while 1:
    target.y = self.y
    if self.distance_to(target) <= limit: break
    tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
    
    if tz < 0.0:
      txz = tx / tz
      
      if   txz < -1.0: yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      elif txz >  1.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      elif self.on_ground and (ty > 3.0) and (tz > -10.0):
        if tz > -2.0: # Reculer pour mieux sauter...
          for i in range(13):
            tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
            txz = tx / tz
            if   txz < -0.3: yield State(self, 0.0, 0.0, 0.2 * self.move_speed, -4.0, "recule", auto_jump = ty > -1.0)
            elif txz >  0.3: yield State(self, 0.0, 0.0, 0.2 * self.move_speed,  4.0, "recule", auto_jump = ty > -1.0)
            else:            yield State(self, 0.0, 0.0, 0.2 * self.move_speed,  0.0, "recule", auto_jump = ty > -1.0)
        yield State(self, 0.0, 0.5, 0.0, 0.0, auto_jump = 1)
      elif txz < -0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed, -4.0, "marche", auto_jump = ty > -1.0)
      elif txz >  0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  4.0, "marche", auto_jump = ty > -1.0)
      else:            yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  0.0, "marche", auto_jump = ty > -1.0)
      
    else:
      if tx < 0.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      else:        yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      
def LookAt(self, target, limit = 0.3):
  while 1:
    tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
    
    if tz < 0.0:
      txz = tx / tz
      
      if   txz < -limit: yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      elif txz >  limit: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      else: return
    else:
      if tx < 0.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      else:        yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      
def Gotoward(self, target, duration, limit = 1.0):
  i = 0
  while (i < duration) and (self.distance_to(target) > limit):
    i+= 1
    tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
    
    if tz < 0.0:
      txz = tx / tz
      
      if   txz < -1.0: yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      elif txz >  1.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      elif self.on_ground and (ty > 3.0) and (tz > -10.0):
        if tz > -2.0: # Reculer pour mieux sauter...
          for i in range(13):
            tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
            txz = tx / tz
            if   txz < -0.3: yield State(self, 0.0, 0.0, 0.2 * self.move_speed, -4.0, "recule", auto_jump = ty > -1.0)
            elif txz >  0.3: yield State(self, 0.0, 0.0, 0.2 * self.move_speed,  4.0, "recule", auto_jump = ty > -1.0)
            else:            yield State(self, 0.0, 0.0, 0.2 * self.move_speed,  0.0, "recule", auto_jump = ty > -1.0)
        yield State(self, 0.0, 0.5, 0.0, 0.0, auto_jump = 1)
      elif txz < -0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed, -4.0, "marche", auto_jump = ty > -1.0)
      elif txz >  0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  4.0, "marche", auto_jump = ty > -1.0)
      else:            yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  0.0, "marche", auto_jump = ty > -1.0)
      
    else:
      if tx < 0.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      else:        yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")

def Fly(self, target, duration = 50):
  for i in range(duration):
    tx, ty, tz = self.transform_point(target.x, target.y, target.z, target.parent)
    
    if tz > 0.0:
      txz = tx / tz
      
      if   txz < -1.0: yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")
      elif txz >  1.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      elif txz < -0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed, -4.0, "marche", auto_jump = ty > -1.0)
      elif txz >  0.3: yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  4.0, "marche", auto_jump = ty > -1.0)
      else:            yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  0.0, "marche", auto_jump = ty > -1.0)
      
    else:
      if tx > 0.0: yield State(self, 0.0, 0.0, 0.0,  4.0, "tourneG")
      else:        yield State(self, 0.0, 0.0, 0.0, -4.0, "tourneD")

def DropItem(self, item):
  yield DropItemState(self, item = item)
  
def GrabItem(self, item):
  yield Goto(self, item)
  if not item.owner: # Else someone already grab it !
    yield GrabItemState(self, item = item)
    
    
class Controler:
  def __init__(self, character):
    self.character = character
    
  def next(self): raise StopIteration
  
  def set_character(self, character):
    self.character = character
    
  def __getstate__(self):
    state = self.__dict__.copy()
    del state["character"] # DO NOT send the character
    return state


class StackControler(Controler):
  def __init__(self, character, controlers  = None):
    Controler.__init__(self, character)
    self.controlers = controlers or []
    self.uncancelable_controler = None
    
  def animate(self, controler):
    if not self.uncancelable_controler in self.controlers:
      self.controlers.append(controler)
      self.uncancelable_controler = controler
      
  def cancel(self, controler):
    try: self.controlers.remove(controler)
    except: return 0
    if self.uncancelable_controler is controler: self.uncancelable_controler = None
    return 1
  
  def animate_and_cancel(self, controler):
    #if not self.uncancelable_controler in self.controlers:
    self.controlers *= 0
    self.controlers.append(controler)
    self.uncancelable_controler = controler
    return 1
  
  def append(self, controler):
    if not self.uncancelable_controler in self.controlers:
      self.controlers.append(controler)
      
  def append_and_cancel(self, controler):
    if not self.uncancelable_controler in self.controlers:
      self.controlers *= 0
      self.controlers.append(controler)
      
  def aaaaaaaaaaaa_next(self):
    while self.controlers:
      try: state = self.controlers[-1].next()
      except StopIteration:
        if self.uncancelable_controler is self.controlers[-1]: self.uncancelable_controler = None
        del self.controlers[-1]
      else:
        if isinstance(state, State): return state
        self.controlers.append(state)
        
  def next(self):
    while self.controlers:
      state = self.controlers[-1]
      if isinstance(state, State):
        del self.controlers[-1]
        return state
      
      try: state = state.next()
      except StopIteration:
        if self.uncancelable_controler is self.controlers[-1]: self.uncancelable_controler = None
        del self.controlers[-1]
      else:
        if isinstance(state, State): return state
        self.controlers.append(state)

  def relation_changed(self, character):
    pass
  
#   def __getstate__(self):
#     state = self.__dict__.copy()
#     state["controlers"] = filter(lambda controler: not type(controler) is types.GeneratorType, self.controlers)
#     if type(self.uncancelable_controler) is types.GeneratorType: state["uncancelable_controler"] = None
#     return state
  
  
TARGETED_ATMOSPHERE = soya.Atmosphere()
TARGETED_ATMOSPHERE.ambient = (1.0, 1.0, 1.0, 1.0)
DATE_TIME = _("__date_time__")

class KeyboardMouseControler(StackControler):
  def __init__(self, character):
    StackControler.__init__(self, character)
    self.left_key_down   = self.right_key_down = self.up_key_down = self.down_key_down = 0
    self.mouse_x         = self.mouse_y = 0
    self.rotation_speed  = 1.0
    self.target          = self.subtarget = None
    self.menu            = None
    self.enemies         = []
    self.doing           = None
    self.doing_activity  = None
    
    self.mouse           = Point()
    self.mouse_dir       = Vector()
    self.location        = Point()
    self.location_normal = Vector()
    self.target_wrapper  = soya.World()
    self.target_wrapper.atmosphere = TARGETED_ATMOSPHERE

    self.mouse_x_pixel   = 0
    self.mouse_y_pixel   = 0
    self.fight_ready     = 0
    self.has_striken     = 0
    
  def is_enemy(self, character):
    if character.name in self.enemies: return 1
    return self.character.alignment * character.alignment < 0
  
  def init(self):
    self.status_bar = StatusBar(soya.root_widget, self.character)

  def find_fight_target(self):
    target = None
    best_dist = 10000.0
    for s in self.character.level:
      if   isinstance(s, base.Strikeable) or (isinstance(s, Character) and (not s is self.character) and (s.life > 0.0) and self.character.is_enemy(s)):
        dist = self.character.distance_to(s)
        if isinstance(s, Character): dist = dist / 2.0 # Characters are targetted prioritary than mushrooms !
        if dist < best_dist:
          best_dist = dist
          target    = s
    return target

  def auto_strike(self):
    if (not self.has_striken) and (not self.character.current_animation.startswith("combat")):
      target = self.find_fight_target()
      if target:
        if isinstance(target, Character):
          soya.IDLER.camera.travelings[0].set_enemy(target)
          
        _P.clone(target)
        _P.convert_to(self.character)
        #p = target % self.character
        c = _P.x / abs(_P.z)
        if self.character.weapon: range = self.character.range + self.character.weapon.range
        else:                     range = self.character.range
        if (_P.z < -1.0 - range) and (-0.6 < c < 0.6):
          self.append_and_cancel(self.character.strike_charge(target))
        elif c < -0.6: self.append_and_cancel(self.character.strike_left  (target))
        elif c >  0.6: self.append_and_cancel(self.character.strike_right (target))
        elif not -1.0 < _P.y < 1.0: self.append_and_cancel(self.character.strike_sagittal(target))
        elif c < 0.0:  self.append_and_cancel(self.character.strike_left  (target))
        else:          self.append_and_cancel(self.character.strike_right (target))
      else:            self.append_and_cancel(self.character.strike_charge(target))
      
  def pause(self):
    import time
    paused = 1
    pause_label = soya.widget.Label(soya.root_widget, _("Paused"), 1, resize_style = ("maximize",))
    
    soya.render()
    while paused:
      time.sleep(0.1)
      for event in soya.process_event():
        if    event[0] == soyasdlconst.KEYDOWN:
          if   event[1] == soyasdlconst.K_p:
            paused = 0
          elif event[1] == soyasdlconst.K_F8:
            import tempfile
            screenshot_file = tempfile.mktemp(".png")
            soya.screenshot(screenshot_file)
            print "* Balazar * Screenshot saved as %s." % screenshot_file
            
        elif (event[0] == soyasdlconst.JOYBUTTONDOWN):
          paused = 0
          
    soya.IDLER.reset()
    soya.root_widget.remove(pause_label)
    
  def next(self):
    need_check_target = 0
    jump = 0
    traveling = soya.IDLER.camera.travelings[0]
    
    for event in soya.process_event():
      if   event[0] == soyasdlconst.JOYAXISMOTION:
        for w in ACTIVE_WIDGETS:
          key = None
          if   event[1] == 0:
            if   event[2] < 0: key = soyasdlconst.K_LEFT
            elif event[2] > 0: key = soyasdlconst.K_RIGHT
          elif event[1] == 1:
            if   event[2] < 0: key = soyasdlconst.K_UP
            elif event[2] > 0: key = soyasdlconst.K_DOWN
            
          if key and w.key_down(key): break
        else:
          if self.fight_ready:
            if   event[1] == 0:
              if   event[2] <  0: self.append_and_cancel(self.character.strike_left ()); self.has_striken = 1
              elif event[2] == 0: self.left_key_down = 0; self.right_key_down = 0
              else:               self.append_and_cancel(self.character.strike_right()); self.has_striken = 1
            elif event[1] == 1:
              if   event[2] <  0: self.append_and_cancel(self.character.strike_sagittal()); self.has_striken = 1
              elif event[2] == 0: self.up_key_down = 0; self.down_key_down = 0
              else:               pass
          else:
            if   event[1] == 0:
              if   event[2] <  0: self.left_key_down = 1; self.right_key_down = 0; self.rotation_speed = 1.0
              elif event[2] == 0: self.left_key_down = 0; self.right_key_down = 0; self.rotation_speed = 1.0
              else:               self.left_key_down = 0; self.right_key_down = 1; self.rotation_speed = 1.0
            elif event[1] == 1:
              if   event[2] <  0: self.up_key_down = 1; self.down_key_down = 0
              elif event[2] == 0: self.up_key_down = 0; self.down_key_down = 0
              else:               self.up_key_down = 0; self.down_key_down = 1
              
      elif event[0] == soyasdlconst.JOYBUTTONDOWN:
        for w in ACTIVE_WIDGETS:
          key = None
          if   event[1] == 0: key = soyasdlconst.K_ESCAPE
          elif event[1] == 1: key = soyasdlconst.K_ESCAPE
          elif event[1] == 2: key = soyasdlconst.K_DELETE
          elif event[1] == 3: key = soyasdlconst.K_RETURN
          elif event[1] == 4: key = soyasdlconst.K_LEFT
          elif event[1] == 5: key = soyasdlconst.K_RIGHT
          elif event[1] == 6: key = soyasdlconst.K_LEFT
          elif event[1] == 7: key = soyasdlconst.K_RIGHT
          if key and w.key_down(key): break
        else:
          if   event[1] == 0:
            if self.fight_ready: self.append_and_cancel(self.character.strike_charge()); self.has_striken = 1
            else: jump = 1
          elif event[1] == 1:
            if self.doing_activity and self.doing_activity.next_activity:
              for activity in self.doing_target.target_activities:
                if (not activity is self.doing_activity) and activity.can_target(self.character, self.doing_target):
                  activity.do(self.character, self.doing_target)
                  break
            else:
              self.fight_ready = 1
              self.has_striken = 0
              target = self.find_fight_target()
              if target:
                if isinstance(target, Character): traveling.set_enemy(target)
          elif event[1] == 2:
            if self.character.left_hand_item:
              for activity in self.character.left_hand_item.target_activities:
                if activity.can_target(self.character, self.character.left_hand_item):
                  self.doing_target   = self.character.left_hand_item
                  self.doing_activity = activity
                  self.doing = activity.do(self.character, self.character.left_hand_item)
                  break
          elif event[1] == 3:
            self.status_bar.set_widget(Inventory(self.character), soya.get_screen_height() / 4.0)
            
          elif event[1] == 5: self.pause()
            
          elif event[1] == 6:
            if not traveling.is_in_fight_mode(): traveling.lateral_angle = -1.8
            
          elif event[1] == 7:
            if not traveling.is_in_fight_mode(): traveling.lateral_angle =  1.8
            
          
          
      elif event[0] == soyasdlconst.JOYBUTTONUP:
        if   event[1] == 0: jump = -1
        elif event[1] == 1:
          self.fight_ready = 0
          self.auto_strike()
        elif event[1] == 2:
          if self.doing:
            self.cancel(self.doing)
            self.doing = self.doing_activity = None
            
        elif event[1] == 6: traveling.lateral_angle = -0.0
        elif event[1] == 7: traveling.lateral_angle = -0.0
            
      elif event[0] == soyasdlconst.MOUSEMOTION:
        self.mouse_x_pixel = event[1]
        self.mouse_y_pixel = event[2]
        
        need_check_target = 1
        
      elif event[0] == soyasdlconst.KEYDOWN:
        if event[1] == 0: continue # My keyboard is buggy
        
        for w in ACTIVE_WIDGETS:
          if w.key_down(event[1]): break
        else:
          if self.subtarget: self.set_subtarget(None)
          #if self.controlers and not self.uncancelable_controler: self.controlers *= 0 # Cancel controlers
          
          if   event[1] == soyasdlconst.K_LEFT:
            if event[2] & soyasdlconst.MOD_CTRL: self.append_and_cancel(self.character.strike_left()); self.has_striken = 1
            else: self.left_key_down = 1; self.right_key_down = 0; self.rotation_speed = 1.0
          elif event[1] == soyasdlconst.K_RIGHT:
            if event[2] & soyasdlconst.MOD_CTRL: self.append_and_cancel(self.character.strike_right()); self.has_striken = 1
            else: self.left_key_down = 0; self.right_key_down = 1; self.rotation_speed = 1.0
          elif event[1] == soyasdlconst.K_UP:
            if event[2] & soyasdlconst.MOD_CTRL: self.append_and_cancel(self.character.strike_sagittal()); self.has_striken = 1
            else: self.up_key_down   = 1
          elif event[1] == soyasdlconst.K_DOWN:   self.down_key_down  = 1
          elif event[1] == soyasdlconst.K_LSHIFT:
            if event[2] & soyasdlconst.MOD_CTRL: self.append_and_cancel(self.character.strike_charge()); self.has_striken = 1
            else: jump = 1
          elif event[1] == soyasdlconst.K_LCTRL:
            if self.doing_activity and self.doing_activity.next_activity:
              for activity in self.doing_target.target_activities:
                if (not activity is self.doing_activity) and activity.can_target(self.character, self.doing_target):
                  activity.do(self.character, self.doing_target)
                  break
            else:
              self.fight_ready = 1
              self.has_striken = 0
              target = self.find_fight_target()
              if target:
                if isinstance(target, Character): traveling.set_enemy(target)
                
          elif event[1] == soyasdlconst.K_LALT:
            if self.character.left_hand_item:
              for activity in self.character.left_hand_item.target_activities:
                if activity.can_target(self.character, self.character.left_hand_item):
                  self.doing_target   = self.character.left_hand_item
                  self.doing_activity = activity
                  self.doing = activity.do(self.character, self.character.left_hand_item)
                  break
                
          elif event[1] == soyasdlconst.K_RETURN: # Inventory
            self.status_bar.set_widget(Inventory(self.character), soya.get_screen_height() / 4.0)
            
          elif (event[1] == soyasdlconst.K_q) or (event[1] == soyasdlconst.K_ESCAPE):
            soya.IDLER.end_game()

          elif event[1] == soyasdlconst.K_F9:
            print self.character.level.X, self.character.level.Y, ":"
            for k, v in self.character.level.id2element.items():
              #s = repr(v)
              #if isinstance(s, unicode): s = s.encode("latin")
              print "ID %s : %s" % (k, v)

          elif event[1] == soyasdlconst.K_h:
            self.character.hurt(0.5, None, 1)
            #self.character.kill(None)

          elif event[1] == soyasdlconst.K_F7: soya.toggle_wireframe()
          elif event[1] == soyasdlconst.K_F8:
            import tempfile
            screenshot_file = tempfile.mktemp(".png")
            soya.screenshot(screenshot_file)
            print "* Balazar * Screenshot saved as %s." % screenshot_file
            
          elif event[1] == soyasdlconst.K_p: self.pause()
      elif event[0] == soyasdlconst.KEYUP:
        if   event[1] == soyasdlconst.K_LEFT:  self.left_key_down  = 0
        elif event[1] == soyasdlconst.K_RIGHT: self.right_key_down = 0
        elif event[1] == soyasdlconst.K_DOWN:
          self.down_key_down = 0
          if self.rotation_speed != 1.0: self.left_key_down = self.right_key_down = 0
        elif event[1] == soyasdlconst.K_UP:
          self.up_key_down = 0
          if self.rotation_speed != 1.0: self.left_key_down = self.right_key_down = 0
        elif event[1] == soyasdlconst.K_LSHIFT: jump = -1
        elif event[1] == soyasdlconst.K_LALT:
          if self.doing:
            self.cancel(self.doing)
            
            #if self.cancel(self.left_hand_doing):
            #  self.left_hand_doing_activity.end_doing(self.character, self.character.left_hand_item)
            
            self.doing = self.doing_activity = None
            
        elif event[1] == soyasdlconst.K_LCTRL:
          self.fight_ready = 0
          self.auto_strike()
            
      elif event[0] == soyasdlconst.MOUSEBUTTONDOWN:
        for w in ACTIVE_WIDGETS:
          if w.mouse_down(event[1], self.mouse_x_pixel, self.mouse_y_pixel): break
        else:
          if   event[1] == 1:
            if self.controlers and not self.uncancelable_controler: self.controlers *= 0 # Cancel controlers
            need_check_target = 2
            
          elif event[1] == 2: self.up_key_down = 2
          
          elif event[1] == 3:
            if self.up_key_down == 2: jump = 1
            else:
              activities = []
              if self.target:
                activities.extend(filter(lambda activity : activity.can_target(self.character, self.target), self.target.target_activities))
              activities.extend(filter(lambda activity : activity.can_target(self.character, self.target), self.character.extra_activities))
              activities.extend(filter(lambda activity : activity.can_target(self.character, self.target), self.character.activities))
              
              if activities:
                activities.sort() # By priority
                #self.menu = Menu(soya.root_widget, self.mouse_x_pixel - 5, self.mouse_y_pixel, activities, self.character, self.target)
                #self.menu.mouse_moved(self.mouse_x_pixel, self.mouse_y_pixel)
                self.show_menu(activities, self.character, self.target)
                
          elif event[1] == 4:
            if (not traveling.is_in_fight_mode()) and (traveling.distance > 4.0):
              traveling.distance -= 1.5
              traveling.top_view -= 0.085

          elif (not traveling.is_in_fight_mode()) and (event[1] == 5):
            if traveling.distance < 18.0:
              traveling.distance += 1.5
              traveling.top_view += 0.085
              traveling.incline_as = None
              
      elif event[0] == soyasdlconst.MOUSEBUTTONUP:
        for w in ACTIVE_WIDGETS:
          if w.mouse_up(event[1], self.mouse_x_pixel, self.mouse_y_pixel): break
        else:
          
          if event[1] == 2: self.up_key_down = 0
          need_check_target = 1 # Mouse Up might have close a panel
          
    if need_check_target:
      for w in ACTIVE_WIDGETS:
        if w.mouse_moved(self.mouse_x_pixel, self.mouse_y_pixel): break
      else:
        soya.IDLER.camera.coord2d_to_3d(self.mouse_x_pixel, self.mouse_y_pixel, result = self.mouse)
        self.mouse_x, self.mouse_y = self.mouse.x, self.mouse.y

        self.check_target()

        if   self.mouse_x < -0.65: traveling.lateral_angle = -0.8
        elif self.mouse_x >  0.65: traveling.lateral_angle =  0.8
        if   (traveling.lateral_angle > 0.0) and self.mouse_x < -0.2: traveling.lateral_angle = 0.0
        elif (traveling.lateral_angle < 0.0) and self.mouse_x >  0.2: traveling.lateral_angle = 0.0

        if self.up_key_down:
          self.rotation_speed = abs(self.mouse_x)
          if   self.mouse_x < 0.0: self.left_key_down = 1; self.right_key_down = 0
          elif self.mouse_x > 0.0: self.left_key_down = 0; self.right_key_down = 1

        else:
          if   self.mouse_x < -0.75: self.left_key_down = 1; self.right_key_down = 0; self.rotation_speed = 1.001
          elif self.mouse_x >  0.75: self.left_key_down = 0; self.right_key_down = 1; self.rotation_speed = 1.001
          elif self.rotation_speed != 1.0: self.left_key_down = self.right_key_down = 0

        if need_check_target == 2:
          if self.location:
            self.append(Goto(self.character, self.location.position()))
          else:
            self.status_bar.text = _("__cant_go__")
          
    if   self.mouse_y >  0.5:
      if traveling.offset_y2 < 4.0: traveling.offset_y2 += (self.mouse_y - 0.5) * 2.0
    elif self.mouse_y < -0.3:
      if traveling.offset_y2 > 0.0: traveling.offset_y2 -= 0.1
      
    if self.controlers:
      state = StackControler.next(self)
      if state and state.animation == "auto":
        if   state.z              < 0.0: state.animation = "marche"
        elif state.z              > 0.0: state.animation = "recule"
        elif state.rotation_speed > 0.0: state.animation = "tourneG"
        elif state.rotation_speed < 0.0: state.animation = "tourneD"
        #else:                            state.animation = "attente"
        elif self.fight_ready:           state.animation = "garde"
        else:                            state.animation = "attente"
    else: state = None
    
    if not state:
      state = State(self.character)
      if   self.fight_ready   : state.animation      = "garde"
      if   jump ==  1         : state.y              =  0.5 ;                             state.animation = "chute"
      elif jump == -1         : state.y              = -0.1 ;                             state.animation = "chute"
      if   self.left_key_down : state.rotation_speed =  4.0  * self.rotation_speed * (2 - (self.up_key_down or self.down_key_down));       state.animation = "tourneG"
      elif self.right_key_down: state.rotation_speed = -4.0  * self.rotation_speed * (2 - (self.up_key_down or self.down_key_down));       state.animation = "tourneD"
      if   self.up_key_down   : state.z              = -0.25 * self.character.move_speed; state.animation = "marche"
      elif self.down_key_down : state.z              =  0.2  * self.character.move_speed; state.animation = "recule"
    elif state.unfinished:
      if state.animation:
        if   jump ==  1         : state.y              =  0.5
        elif jump == -1         : state.y              = -0.1
        if   self.left_key_down : state.rotation_speed =  4.0  * self.rotation_speed
        elif self.right_key_down: state.rotation_speed = -4.0  * self.rotation_speed
        if   self.up_key_down   : state.z              = -0.25 * self.character.move_speed
        elif self.down_key_down : state.z              =  0.2  * self.character.move_speed
      else:
        if   self.fight_ready   : state.animation      = "garde"
        if   jump ==  1         : state.y              =  0.5 ;                             state.animation = "chute"
        elif jump == -1         : state.y              = -0.1 ;                             state.animation = "chute"
        if   self.left_key_down : state.rotation_speed =  4.0  * self.rotation_speed;       state.animation = "tourneG"
        elif self.right_key_down: state.rotation_speed = -4.0  * self.rotation_speed;       state.animation = "tourneD"
        if   self.up_key_down   : state.z              = -0.25 * self.character.move_speed; state.animation = "marche"
        elif self.down_key_down : state.z              =  0.2  * self.character.move_speed; state.animation = "recule"
        
    return self.character.collision_detection(state)
  
  def show_panel(self, text):
    #if self.menu: self.menu.destroy()
    self.menu = Panel(soya.root_widget.main_view, self.mouse_x_pixel - 5, self.mouse_y_pixel, text)
    self.menu.mouse_moved(self.mouse_x_pixel, self.mouse_y_pixel)
    
  def show_menu(self, options, *args):
    #if self.menu: self.menu.destroy()
    self.menu = Menu(soya.root_widget.main_view, self.mouse_x_pixel - 5, self.mouse_y_pixel, options, *args)
    self.menu.mouse_moved(self.mouse_x_pixel, self.mouse_y_pixel)
    
  def big_round(self):
    daynigth = self.character.level.atmosphere
    #self.status_bar.date_text = DATE_TIME % (daynigth.day, int(daynigth.time), int(60 * (daynigth.time % 1)))
    
  def check_target(self):
    target_distance = 1000.0
    
    self.status_bar.selected_character = None

    self.mouse    .__init__(soya.IDLER.camera, self.mouse_x, self.mouse_y, -1.0)
    self.mouse_dir.__init__(soya.IDLER.camera, self.mouse_x, self.mouse_y, -1.0)

    characters = self.character.level.characters.values()
    for character in characters: character.solid = 1 # Temporarily enable raypick on characters
    r = soya.IDLER.scene.raypick(self.mouse, self.mouse_dir, -1.0, 1, 1, self.location, self.location_normal)
    for character in characters: character.solid = 0
    
    if r: self.set_subtarget(self.location.parent)
    else: self.set_subtarget(None)
    
  def set_subtarget(self, subtarget):
    if not subtarget is self.subtarget:
      self.subtarget = subtarget
      if subtarget:
        target = subtarget
        while target and not isinstance(target, base.Entity): target = target.parent

#         display_name = ""
#         if hasattr(subtarget, "display_name"):
#           if callable(subtarget.display_name): display_name = subtarget.display_name()
#           else:                                display_name = subtarget.display_name
#         else:
#           if getattr(subtarget, "shape", None):
#             display_name = _("__shape__%s" % subtarget.shape.filename)
#             if display_name[0] == "_": display_name = ""
#         if not display_name:
#           if hasattr(target, "display_name"):
#             if callable(target.display_name): display_name = target.display_name()
#             else:                             display_name = target.display_name
#           else:
#             if getattr(target, "shape", None):
#               display_name = _("__shape__%s" % target.shape.filename)
#               if display_name[0] == "_": display_name = ""
#              
#         self.status_bar.text = display_name
        
        self.status_bar.text = base.display_name(subtarget)
        
      else:
        self.status_bar.text = ""
        target = None
        
      if not target is self.target:
        if isinstance(self.target , soya.World): self.target.atmosphere = None
        elif self.target and self.target.can_change_parent:
          self.target_wrapper.remove(self.target)
          self.target_wrapper.parent.add(self.target)
          self.target_wrapper.parent.remove(self.target_wrapper)
          
        self.target = target
        
        if isinstance(self.target, soya.World): self.target.atmosphere = TARGETED_ATMOSPHERE
        elif self.target and self.target.can_change_parent:
          self.target.parent.add(self.target_wrapper)
          self.target.parent.remove(self.target)
          self.target_wrapper.add(self.target)
          
import soya.opengl as soyaopengl

    
ACTIVE_WIDGETS = []
class ActiveWidget:
  def __init__(self):
    ACTIVE_WIDGETS.insert(0, self)
    
  def key_down   (self,         key ): """Returns 1 if the event has been consumed"""
  def mouse_moved(self,         x, y): """Returns 1 if the event has been consumed"""
  def mouse_down (self, button, x, y):
    """Returns 1 if the event has been consumed"""
    return self.mouse_moved(x, y)
  def mouse_up   (self, button, x, y):
    """Returns 1 if the event has been consumed"""
    return self.mouse_moved(x, y)
  
  def destroy(self):
    try: self.master.remove(self)
    except: pass
    try: ACTIVE_WIDGETS.remove(self)
    except: pass
    
    
class FocusableWidget(ActiveWidget):
  def __init__(self):
    self.focus = 0
    ActiveWidget.__init__(self)

  def set_focus(self, focus, play_sound = 1):
    self.focus = focus
    if focus and play_sound: sound.play("menu2.wav")
    
  def key_down(self, key):
    if  (key == soyasdlconst.K_ESCAPE) and self.focus: self.set_focus(0)
    elif(key == soyasdlconst.K_DELETE) and self.focus: self.set_focus(0)
    elif key == soyasdlconst.K_RETURN:
      focusable = [widget for widget in ACTIVE_WIDGETS if isinstance(widget, FocusableWidget)]
      if self is focusable[0]:
        for widget in focusable:
          if widget.focus: break
        else:
          self.set_focus(1)
          return 1
      else:
        preceding = focusable[focusable.index(self) - 1]
        if preceding.focus:
          preceding.set_focus(0)
          self     .set_focus(1)
          return 1
      if (self is focusable[-1]) and self.focus:
        self.set_focus(0)
        # Do NOT return 1, => open inventory
        
STATUS_BAR_MATERIAL = soya.Material.get("status_bar")

TEXT_HEIGHT = widget.default_font.get_print_size("JIBA")[1]

class StatusBar(ActiveWidget, widget.Label):
  def __init__(self, master, default_character = None):
    self.fps_label = soya.widget.FPSLabel()
    
    ActiveWidget.__init__(self)
    widget.Label.__init__(self, master)
    
    master.status_bar       = self
    self.widget             = None
    self.extra_height       = 0
    self.selected_character = None
    self.default_character  = default_character
    
    self.old_life = self.old_mana = -1
    
    self.left        = 0
    self.height      = int(TEXT_HEIGHT + 7)
    self.base_height = self.height
    
    
  def mouse_moved(self,         x, y):
    if y >= self.top:
      if not self.widget:
        self.set_widget(Inventory(self.default_character), soya.get_screen_height() / 4.0)
      return 1
    
    elif self.widget and self.extra_height:
      self.widget.destroy()
    
  def mouse_down (self, button, x, y):
    return self.mouse_moved(x, y)
  
  def mouse_up   (self, button, x, y):
    return self.mouse_moved(x, y)
  
  def resize(self, parent_left = 0, parent_top = 0, parent_width = 0, parent_height = 0):
    self.width  = int(soya.get_screen_width())
    self.top    = self.master.main_view.height
    self.height = int(soya.get_screen_height() - self.top)
    self.tex_x  = int(self.width / 115.0)
    
    self.fps_label.resize(0, self.top, self.width, self.height)
    
    
  def set_widget(self, widget, extra_height = 0):
    traveling = soya.IDLER.camera.travelings[0]
    if widget:
      sound.play("menu2.wav")
      if self.widget:
        self.widget.destroy()
        
      self.widget = widget
      widget.master = self
      widget.resize(self.left + 10, self.top + self.base_height, self.width - 20, self.height - self.base_height)
      
      self.text = getattr(self.widget, "display_name", "")
      
      traveling.lateral_angle = 3.14
    else:
      self.text = ""
      if traveling.lateral_angle > 3.0: traveling.lateral_angle = 0
      
    self.extra_height = extra_height - extra_height % 20
    
  def remove(self, widget):
    if widget is self.widget: self.set_widget(None)
    
  def build_display_list(self):
    STATUS_BAR_MATERIAL.activate()
    soyaopengl.glBegin(soyaopengl.GL_QUADS)
    soyaopengl.glTexCoord2f( 0.0, 1.0); soyaopengl.glVertex2i(0, self.top)
    soyaopengl.glTexCoord2f( 0.0, 0.0); soyaopengl.glVertex2i(0, self.top + self.height)
    soyaopengl.glTexCoord2f(self.tex_x, 0.0); soyaopengl.glVertex2i(self.width, self.top + self.height)
    soyaopengl.glTexCoord2f(self.tex_x, 1.0); soyaopengl.glVertex2i(self.width, self.top)
    soyaopengl.glEnd()
    soya.DEFAULT_MATERIAL.activate()
    
    character = self.selected_character or self.default_character
    if character:
      soyaopengl.glColor4f(0.8, 0.0, 0.0, 1.0)
      soyaopengl.glBegin(soyaopengl.GL_QUADS)
      w = int(0.75 * self.width) - 125
      soyaopengl.glVertex2i(w, self.top + 5)
      soyaopengl.glVertex2i(w, self.top + self.base_height / 2)
      w += int(0.25 * character.life * self.width)
      soyaopengl.glVertex2i(w, self.top + self.base_height / 2)
      soyaopengl.glVertex2i(w, self.top + 5)
      soyaopengl.glEnd()

      if character.mana:
        soyaopengl.glColor4f(0.0, 0.0, 0.8, 1.0)
        soyaopengl.glBegin(soyaopengl.GL_QUADS)
        w = int(0.75 * self.width) - 125
        soyaopengl.glVertex2i(w, self.top + self.base_height / 2)
        soyaopengl.glVertex2i(w, self.top + self.base_height - 5)
        w += int(0.25 * character.mana * self.width)
        soyaopengl.glVertex2i(w, self.top + self.base_height - 5)
        soyaopengl.glVertex2i(w, self.top + self.base_height / 2)
        soyaopengl.glEnd()
        
    soyaopengl.glColor4f(1.0, 1.0, 1.0, 1.0)
    self._font.draw(self.text, self.left + 10, self.top)
    
  def render(self):
    widget.Label.render(self)
    
    if self.widget: self.widget.render()
    
    if   self.base_height + self.extra_height != self.height:
      if   self.base_height + self.extra_height > self.height:
        main_view = self.master.main_view
        main_view.resize(main_view.left, main_view.top, main_view.width, main_view.height + self.base_height - 20)
        self     .resize()
      elif self.base_height + self.extra_height < self.height:
        main_view = self.master.main_view
        main_view.resize(main_view.left, main_view.top, main_view.width, main_view.height + self.base_height + 20)
        self     .resize()
        
        if self.widget and (self.height == self.base_height):
          self.widget.master = None
          self.widget = None
          
      if self.widget:
        self.widget.resize(self.left + 10, self.top + self.base_height, self.width - 20, self.height - self.base_height)
        
      self._changed = -2
      
    self.fps_label.render()
      
  def widget_begin_round(self):
    character = self.selected_character or self.default_character
    if (self.old_life != character.life) or (self.old_mana != character.mana):
      self.old_life = character.life
      self.old_mana = character.mana
      self._changed = -2
    if self.widget: self.widget.widget_begin_round()
    
    self.fps_label.widget_begin_round()
    
  def widget_advance_time(self, proportion):
    if self.widget: self.widget.widget_advance_time(proportion)
  def widget_end_round(self):
    if self.widget: self.widget.widget_end_round()
    
    
class Menu(widget.Label, ActiveWidget):
  def __init__(self, master, x, y, options, *args):
    self.x        = int(x)
    self.y        = int(y)
    self.options  = options
    self.args     = args
    self.selected = -1
    ActiveWidget .__init__(self)
    widget.Label.__init__(self, master)
    sound.play("menu2.wav")
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.width  = int(max([self.font.get_print_size(option.display_name)[0] for option in self.options]))
    self.height = int(TEXT_HEIGHT * len(self.options)) + 10
    self.left   = int(min(max(self.x, parent_left), parent_left + parent_width  - self.width ))
    self.top    = int(min(max(self.y, parent_top ), parent_top  + parent_height - self.height))
    self._changed = -2
    
  def mouse_moved(self, x, y):
    if self.left < x < self.left + self.width:
      if y > self.top:
        for i in range(len(self.options)):
          if y <= self.top + (i + 1) * TEXT_HEIGHT:
            if self.selected != i:
              self.selected = i
              self._changed = -2
            return 1
    if self.selected != -1:
      self.selected = -1
      self._changed = -2
      
  def mouse_down(self, button, x, y): return self.mouse_moved(x, y)
  
  def mouse_up(self, button, x, y):
    if self.mouse_moved(x, y):
      sound.play("menu1.wav")
      self.options[self.selected].do(*self.args)
    self.destroy()
    
  def build_display_list(self):
    STATUS_BAR_MATERIAL.activate()
    soyaopengl.glBegin(soyaopengl.GL_QUADS)
    soyaopengl.glTexCoord2f(0.0, 1.0); soyaopengl.glVertex2i(self.left - 4, self.top)
    soyaopengl.glTexCoord2f(0.0, 0.0); soyaopengl.glVertex2i(self.left - 4, self.top + self.height)
    soyaopengl.glTexCoord2f(1.0, 0.0); soyaopengl.glVertex2i(self.left + self.width + 3, self.top + self.height)
    soyaopengl.glTexCoord2f(1.0, 1.0); soyaopengl.glVertex2i(self.left + self.width + 3, self.top)
    soyaopengl.glEnd()
    soya.DEFAULT_MATERIAL.activate()
    
    soyaopengl.glColor4f(0.8, 0.7, 0.8, 1.0)
    for i in range(len(self.options)):
      if i == self.selected:
        soyaopengl.glColor4f(1.0, 1.0, 1.0, 1.0)
        self._font.draw(self.options[i].display_name, self.left, self.top + i * TEXT_HEIGHT)
        soyaopengl.glColor4f(0.8, 0.7, 0.8, 1.0)
      else:
        self._font.draw(self.options[i].display_name, self.left, self.top + i * TEXT_HEIGHT)
        
        
class Panel(widget.Label, ActiveWidget):
  def __init__(self, master, x, y, text):
    self.x        = x
    self.y        = y
    ActiveWidget.__init__(self)
    widget.Label.__init__(self, master, text)
    self.selected = 0
    sound.play("menu2.wav")
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.width, self.height, self._text = self.font.wordwrap(self.text, 500)
    self.width  = int(self.width)
    self.height = int(self.height) + 10
    self.left   = int(min(max(self.x, parent_left), parent_left + parent_width  - self.width ))
    self.top    = int(min(max(self.y, parent_top ), parent_top  + parent_height - self.height))
    self._changed = -2
    
  def mouse_moved(self, x, y):
    selected = (self.left < x <= self.left + self.width) and (self.top < y <= self.top + self.height)
    if self.selected != selected:
      self._changed = -2
      self.selected = selected
    return self.selected
  
  def mouse_down(self, button, x, y):
    return self.mouse_moved(x, y)
    
  def mouse_up(self, button, x, y):
    if self.mouse_moved(x, y):
      sound.play("menu1.wav")
      self.destroy()
      
  def build_display_list(self):
    STATUS_BAR_MATERIAL.activate()
    soyaopengl.glBegin(soyaopengl.GL_QUADS)
    soyaopengl.glTexCoord2f(0.0, 1.0); soyaopengl.glVertex2i(self.left - 4, self.top)
    soyaopengl.glTexCoord2f(0.0, 0.0); soyaopengl.glVertex2i(self.left - 4, self.top + self.height)
    soyaopengl.glTexCoord2f(1.0, 0.0); soyaopengl.glVertex2i(self.left + self.width + 3, self.top + self.height)
    soyaopengl.glTexCoord2f(1.0, 1.0); soyaopengl.glVertex2i(self.left + self.width + 3, self.top)
    soyaopengl.glEnd()
    soya.DEFAULT_MATERIAL.activate()

    if self.selected: soyaopengl.glColor4f(1.0, 1.0, 1.0, 1.0)
    else:             soyaopengl.glColor4f(0.8, 0.7, 0.8, 1.0)

    self._font.draw(self.text, self.left, self.top, 0.0)
    #self._font.draw_area(self.text, self.left, self.top, 0.0, self.width, 100000.0, 0)
      
    
class Inventory(ActiveWidget, widget.Widget, soya.World):
  display_name = _("__activity__ListItems")
  def __init__(self, character, items = None):
    soya.World   .__init__(self)
    ActiveWidget .__init__(self)
    widget.Widget.__init__(self)
    
    self.mouse           = soya.Point()
    self.selection       = None
    
    self.character = character
    self.items     = items or character.items
    self.camera    = soya.Camera(self)
    self.camera.z  = 2.0
    self.camera.set_xyz(0.0, 2.6, 6.5)
    self.camera.resize(0, 0, soya.get_screen_width(), soya.get_screen_height())
    
    self.light     = soya.Light(self)
    self.light.set_xyz(3.0, 2.0, 1.0)
    self.light.diffuse = (0.8, 0.8, 0.8, 1.0)
    
    self.atmosphere = soya.NoBackgroundAtmosphere()
    self.atmosphere.ambient = (0.2, 0.0, 0.2, 1.0)
    
    self.focus = soya.World(self)
    self.focus.atmosphere = soya.NoBackgroundAtmosphere()
    self.focus.atmosphere.ambient = (2.0, 2.0, 2.0, 1.0)
    
    self.clones = []
    self.old_items = []
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.left   = parent_left
    self.top    = parent_top
    self.width  = parent_width
    self.height = parent_height
    
    self.camera.y = 2.6 + (soya.get_screen_height() / 4.0 - parent_height) / 60.0
    
    #self.camera.resize(parent_left, parent_top, parent_width, parent_height)
    #self.camera.resize(0, 200, 600, 200)

  def render(self):
    self.camera.render()

  def widget_begin_round(self):
    import copy
    if self.old_items != self.items:
      # self.items have changed !!!
      self.select_item(None)
      
      d = dict([(clone.item, clone) for clone in self.clones])
      
      for clone in self.clones[:]:
        if not clone.item.owner is self.character:
          clone.parent.remove(clone)
          
          
      self.clones = []
      x = -0.55 * (len(self.items) - 1)
      for item in self.items:
        clone = d.get(item)
        if not clone:
          #clone = item.__class__(self)
          clone = copy.deepcopy(item)
          self.add(clone)
          clone.item = item
          clone.set_inventory_pos()
        else:
          clone.zap()
        clone.x = x
        x += 1.1
        
        self.clones.append(clone)
        
      self.old_items = self.items[:]
      
      
  def widget_advance_time(self, proportion):
    #for clone in self.clones:
    #  clone.rotate_lateral(1.0)
    if self.selection and not isinstance(self.selection, Photo): self.selection.rotate_lateral(4.0)
    
  def mouse_moved(self, x, y):
    if y < self.top: return 0
    
    self.camera.coord2d_to_3d(x, y, -self.camera.z, self.mouse)
    self.mouse.x += self.camera.x
    
    i = int((self.mouse.x / 1.1) + len(self.items) / 2.0)
    if 0 <= i < len(self.items):
      if not self.clones[i] is self.selection: self.select_item(self.clones[i])
    else:
      self.select_item(None)
      
    return 1
  
  def select_item(self, item):
    if self.selection:
      self.focus.remove(self.selection)
      self.add(self.selection)
      
    self.selection = item
    if self.selection:
      self.remove(self.selection)
      self.focus.add(self.selection)
      self.master.text = base.display_name(self.selection)
      
      if   -self.camera.x + self.selection.x >  4.0: self.camera.x = self.selection.x - 4.0
      elif -self.camera.x + self.selection.x < -4.0: self.camera.x = self.selection.x + 4.0
    else:
      self.master.text = _("__activity__ListItems")

  def validate(self):
    sound.play("menu1.wav")
    item = self.selection.item
    if isinstance(item, EquipableItem):
      item.set_equiped(not item.equiped)
    else:
      for activity in item.target_activities:
        if activity.can_target(self.character, item):
          activity.do(self.character, item)
          break
        
  def mouse_down(self, button, x, y):
    if self.mouse_moved(x, y):
      if   (button == 1) and self.selection: self.validate()
      
      elif (button == 3) and self.selection:
        item = self.selection.item
        activities = []
        activities.extend(filter(lambda activity : activity.can_target(self.character, item), item.target_activities))
        activities.extend(filter(lambda activity : activity.can_target(self.character, item), self.character.extra_activities))
        activities.extend(filter(lambda activity : activity.can_target(self.character, item), self.character.activities))
        
        activities.sort() # By priority
        self.menu = Menu(soya.root_widget, x - 5, y, activities, self.character, item)
        self.menu.mouse_moved(x, y)
        
      return 1
    
  def mouse_up(self, button, x, y):
    if self.mouse_moved(x, y): return 1
    
  def key_down(self, key):
    if   key == soyasdlconst.K_LEFT:
      if not self.selection: self.select_item(self.clones[len(self.clones) // 2])
      else:
        i = self.clones.index(self.selection)
        if i > 0: self.select_item(self.clones[i - 1])
      return 1
    
    elif key == soyasdlconst.K_RIGHT:
      if not self.selection: self.select_item(self.clones[len(self.clones) // 2])
      else:
        i = self.clones.index(self.selection)
        if i < len(self.clones) - 1: self.select_item(self.clones[i + 1])
      return 1
    
    elif key == soyasdlconst.K_RETURN:
      if self.selection: self.validate()
      self.destroy()
      return 1
    
    elif key == soyasdlconst.K_DELETE:
      sound.play("menu1.wav")
      if self.selection:
        self.character.queue.controler.append_and_cancel(DropItem(self.character, self.selection.item))
      return 1
    
    elif (key == soyasdlconst.K_ESCAPE) or (key == soyasdlconst.K_q):
      sound.play("menu1.wav")
      self.destroy()
      return 1
 
from balazar.item import photo_model

   
class PhotoViewer(ActiveWidget, widget.Label, soya.World):
  def __init__(self, photo):
    soya.World   .__init__(self)
    ActiveWidget .__init__(self)
    widget.Label .__init__(self)
    
    from balazar.item import photo_model
    
    self.photo = photo
    
    photo_model.children[0].material = soya.Material(soya.image_from_pil(photo.image.resize((1024, 512))))
    self.volume = soya.Volume(self, photo_model.shapify())
    photo_model.children[0].material = soya.DEFAULT_MATERIAL
    
    self.camera = soya.Camera(self)
    self.camera.set_xyz(0.0, 10.0, 0.85)
    self.camera.resize(0, 0, soya.get_screen_width(), soya.get_screen_height())
    
    self.atmosphere = soya.NoBackgroundAtmosphere()
    self.atmosphere.ambient = (1.0, 1.0, 1.0, 1.0)
    
    self.text = photo.full_display_name()
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.left   = parent_left
    self.top    = parent_top
    self.width  = parent_width
    self.height = parent_height
    
    #self.camera.y = -0.017 + (soya.get_screen_height() / 1.2 - (parent_height - int(TEXT_HEIGHT + 7))) / 500.0
    self.camera.y = 0.09 + (soya.get_screen_height() / 1.2 - (parent_height - int(TEXT_HEIGHT + 7))) / 500.0
    self._changed = -2
    
  def render(self):
    self.camera.render()
    widget.Label.render(self)
    
  def mouse_moved(self, x, y):
    if y < self.top: return 0
    return 1
  
  def mouse_down(self, button, x, y):
    if self.mouse_moved(x, y):
      sound.play("menu1.wav")
      self.destroy()
      return 1
    
  def key_down(self, key):
    self.destroy()
    return 1
   
    
