# Balazar
# Copyright (C) 2005 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import balazar.base  as base
import balazar.sound as sound
from balazar.controler import FocusableWidget

import soya
import soya.widget   as widget
import soya.opengl   as soyaopengl
import soya.sdlconst as soyasdlconst


VOICE = None
class Discussion(object):
  def __init__(self, character, reply, text = u"", condition = 1, action = None, check = None, source = None, voice = None, choices = ()):
    self.character   = character
    self.reply       = reply
    self.text        = text
    self.condition   = condition
    self.action      = action
    self.check       = check
    self.source      = source or character
    self.voice       = voice
    
    if not callable(choices): self.set_choices(choices)
    else:                     self._choices = choices
    #if choices:
    #  if not callable(choices): self.set_choices(choices)
    #  else:                     self._choices = choices
    #elif self.text:
    
  def activate(self, focus = 0):
    if self.action: self.action()
    if self.text:
      global VOICE
      if VOICE: VOICE.stop()
      try: VOICE = sound.play(self.voice or (u"voice_%s.ogg" % " ".join(self.text.split()[:4])).encode("latin"))
      except: pass # No voice (for this discussion, or at all)
      discussion_widget = DiscussionWidget(soya.root_widget, self)
      if focus:
        discussion_widget.set_focus(1, 0)
        if discussion_widget.choices: discussion_widget.selection = discussion_widget.choices[0][0]

  def is_valid(self):
    if callable(self.condition): return self.condition()
    return self.condition
  
  def set_choices(self, choices):
    self._choices = []
    for choice in choices:
      if choice.is_valid():
        if self._choices and (self._choices[-1].reply == choice.reply): # Same text => replace
          self._choices[-1] = choice
        else:
          self._choices.append(choice)
          
    if self.text and not self._choices:
      self.set_choices([Discussion(self.character, _("Ok"))])
      
  def get_choices(self):
    if callable(self._choices): self.set_choices(self._choices())
    return self._choices
  choices = property(get_choices)
  
  

def StartDiscussion(hero):
  return Discussion(hero, u"", _(u"\"Un monde trange s'ouvrait devant moi. Je devais \
retrouver les anciennes merveilles, et leur redonner vie grce  mon appareil...\""),
                    choices = [
    Discussion(hero, _(u"Suite (ENTRE ou le BOUTON 4 du joypad)"), _(u"Cela, je l'crirai ce soir dans mon journal. \
Pour l'instant je dois retrouver l'ancien temple et le prendre en photo avant qu'il \
ne disparaisse."), choices = [
      Help(hero, _(u"Aide (ENTRE ou le BOUTON 4 du joypad)")),
      Discussion(hero, _(u"Je sais dj jouer !")),
      ]),
    ])

def Help(hero, reply = u""):
  return Discussion(hero, reply,
                    _(u"Utilise les touches HAUT BAS GAUCHE DROITE ou la MANETTE du \
joypad pour te dplacer"), choices = [
    Discussion(hero,
               _(u"Suite (ENTRE ou le BOUTON 4 du joypad)"),
               _(u"Utilise la touche SHIFT ou le BOUTON 1 du joypad pour sauter. Appuie \
sur ENTRE ou le BOUTON 4 du joypad pour revenir  l'aide, puis en BAS pour \
slectionner 'Suite'."), choices = [
      Discussion(hero,
                 _(u"Suite"),
                 _(u"Appuis sur la touche CONTRLE ou le BOUTON 2 du joypad pour te \
mettre en garde. Relche le bouton pour frapper."), choices = [
        Discussion(hero,
                   _(u"Suite"),
                   _(u"Utilise la touche ALT ou le BOUTON 3 du joypad pour activer \
l'objet en main gauche. Appuis en mme temps sur la touche CONTRLE ou le BOUTON 2 \
du joypad pour prendre une photo."), choices = [
          Discussion(hero,
                     _(u"Suite"),
                     _(u"Utilise les touches HAUT BAS GAUCHE DROITE ou la MANETTE du \
joypad pour te dplacer  l'intrieur des fentres. Utilise la touche ENTRE ou le \
BOUTON 4 du joypad pour valider ou pour changer de fentre."), choices = lambda : [
            Help(hero, _(u"Reprendre l'aide au dbut")),
            Discussion(hero, _(u"Fin de l'aide")),
            ]),
          ]),
        ]),
      ]),
    ])


class DiscussionScript(soya.CoordSyst, base.Discutable):
  def __init__(self, parent = None, discussion = None, radius = 5.0):
    soya.CoordSyst.__init__(self, parent)
    self.discussion        = discussion
    self.discussion_radius = radius
    if not self.discussion.source: self.discussion.source = self
    
  def start_discussion(self, hero):
    from balazar.discussion import Discussion
    from balazar.controler  import GrabItem
    if self.discussion.is_valid(): self.discussion.activate()
    
    
    
CURRENT_DISCUSSION = None

class DiscussionWidget(widget.Label, FocusableWidget):
  def __init__(self, master, discussion):
    FocusableWidget.__init__(self)
    self.discussion = discussion
    self.character  = discussion.character
    if not self.character:
      from balazar.player import CURRENT_CHARACTER
      self.character = CURRENT_CHARACTER
      
    widget.Label.__init__(self, master)
    self.left       = 4
    self.top        = 0
    self.selection  = None
    self.position   = soya.Point()
    self.focus      = 0
    self.visible    = 1
    
    self.width, self.text_height, self._text = self.font.wordwrap(discussion.text, 500)
    self.height  = self.text_height + 20
    if discussion.choices: self.height += 10
    self.choices = []
    for choice in discussion.choices:
      width, height, text = self.font.wordwrap(choice.reply, 500)
      self.choices.append((choice, height, text))
      if width > self.width: self.width = width
      self.height += height
    self.width  = int(self.width ) + 28
    self.height = int(self.height)
    
    global CURRENT_DISCUSSION
    CURRENT_DISCUSSION = self
    
  def destroy(self):
    global CURRENT_DISCUSSION
    CURRENT_DISCUSSION = None
    
    FocusableWidget.destroy(self)

  def check(self):
    if self.discussion.check and not self.discussion.check(): return 0
    from balazar.player import CURRENT_CHARACTER
    if CURRENT_CHARACTER.distance_to(self.discussion.source) > self.discussion.source.discussion_radius: return 0
    return 1
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    pass
    
  def set_focus(self, focus, play_sound = 1):
    FocusableWidget.set_focus(self, focus, play_sound)
    if focus == 0: self.selection = None
    self._changed = -2
    
  def mouse_moved(self, x, y):
    y2 = self.text_height + 4
    for choice, height, text in self.choices:
      y2 += height
      if y <= y2:
        if not self.selection is choice:
          self.selection = choice
          self._changed = -2
        return 1
    else:
      if self.selection:
        self.selection = None
        self._changed = -2
        
  def mouse_down(self, button, x, y):
    return self.mouse_moved(x, y)
    
  def mouse_up(self, button, x, y):
    if self.mouse_moved(x, y): self.choose(self.selection)
    
  def key_down(self, key):
    if self.focus:
      if   (key == soyasdlconst.K_UP  ) and self.choices and ((not self.selection) or (len(self.choices) > 1)):
        if self.selection: self.selection = self.choices[self.discussion.choices.index(self.selection) - 1][0]
        else:              self.selection = self.choices[-1][0]
        self._changed = -2
        return 1
      elif (key == soyasdlconst.K_DOWN) and self.choices and ((not self.selection) or (len(self.choices) > 1)):
        if self.selection: self.selection = self.choices[(self.discussion.choices.index(self.selection) + 1) % len(self.discussion.choices)][0]
        else:              self.selection = self.choices[ 0][0]
        self._changed = -2
        return 1
      elif (key == soyasdlconst.K_RETURN) and self.selection:
        self.choose(self.selection)
        return 1
      
    return FocusableWidget.key_down(self, key)
  
  def choose(self, choice):
    sound.play("menu1.wav")
    self.destroy()
    choice.activate(1)
    
  def build_display_list(self):
    soyaopengl.glColor4f(0.0, 0.0, 0.0, 1.0)
    self._font.draw(self.text, 14, 5, 0)
    
    y = self.text_height + 15
    for choice, height, text in self.choices:
      if choice is self.selection: soyaopengl.glColor4f(0.0, 0.0, 1.0, 1.0)
      else:                        soyaopengl.glColor4f(0.5, 0.5, 0.5, 1.0)
      self._font.draw(text, 14, y, 0)
      y += height
      
  def render(self):
    if not self.visible: return
    
    self.position.__init__(self.character, 0.0, 2.6, 0.0)
    self.position.convert_to(soya.IDLER.camera)
    if self.position.z > 0.0: return
    self.position.__init__(self.character, 0.0, 2.6, 0.0)
    x, y = x2, y2 = soya.IDLER.camera.coord3d_to_2d(self.position)
    
    if x2 < soya.IDLER.camera.width / 2 - 30: x2 = self.width / 2.0
    else:                                     x2 = soya.IDLER.camera.width - self.width / 2.0
    y2 = self.height
    #if   x2 < self.width / 2.0: x2 = self.width / 2.0
    #elif x2 > soya.IDLER.camera.width - self.width / 2.0: x2 = soya.IDLER.camera.width - self.width / 2.0
    #y2 -= 50
    #if   y2 < self.height: y2 = self.height
    #elif y2 > soya.IDLER.camera.height: y2 = soya.IDLER.camera.height
    
    soyaopengl.glPushMatrix()
    soyaopengl.glTranslatef(x2 - self.width / 2.0, y2 - self.height, 0.0)
    
    soya.DEFAULT_MATERIAL.activate()
    soyaopengl.glEnable(soyaopengl.GL_BLEND)
    if self.focus or self.selection: soyaopengl.glColor4f(1.0, 1.0, 1.0, 1.0)
    else:                            soyaopengl.glColor4f(1.0, 1.0, 1.0, 0.7)
    soyaopengl.glBegin(soyaopengl.GL_POLYGON)
    soyaopengl.glVertex2i(0, 0)
    if x2 == self.width / 2.0:
      soyaopengl.glVertex2i(0              , self.height)
      soyaopengl.glVertex2i(self.width - 15, self.height)
      soyaopengl.glVertex2i(self.width - 5 , self.height -  5)
      soyaopengl.glVertex2i(self.width     , self.height - 15)
      
    else:
      soyaopengl.glVertex2i(0         , self.height - 15)
      soyaopengl.glVertex2i(5         , self.height -  5)
      soyaopengl.glVertex2i(15        , self.height)
      soyaopengl.glVertex2i(self.width, self.height)
      
    soyaopengl.glVertex2i(self.width, 0)
    soyaopengl.glEnd()
    if 1:
      soyaopengl.glBegin(soyaopengl.GL_TRIANGLES)
      soyaopengl.glVertex2i(int(self.width * 0.4), self.height)
      soyaopengl.glVertex2i(int(self.width * 0.3), self.height)
      soyaopengl.glColor4f(1.0, 1.0, 1.0, 0.1)
      soyaopengl.glVertex2i(int(x - x2 + self.width / 2.0), int(y - y2 + self.height))
      soyaopengl.glEnd()
    soyaopengl.glDisable(soyaopengl.GL_BLEND)
    
    widget.Label.render(self)
    soyaopengl.glPopMatrix()

