# Balazar
# Copyright (C) 2003 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from balazar.character import *
from balazar.character import _P, _V
import balazar.base       as base
import balazar.activity   as activity
import balazar.sound      as sound
import balazar.discussion as discussion


QUALITIES = [
  "brave",
  "coward",
  "resistant",
  "ascetic",
  "greedy",
  "hatesorcerer",
  "lovemagic",
  ]

QUALITY_CONTRARY = {
  "brave"        : "coward",
  "coward"       : "brave",
  "ascetic"      : "greedy",
  "greedy"       : "ascetic",
  "hatesorcerer" : "lovemagic",
  "lovemagic"    : "hatesorcerer",
  }

class DretzelCandidate(Character):
  alignment = 1
  move_speed  = 0.8
  
  def __init__(self, player = 0):
    Character.__init__(self, player)
    self.qualities = {}
    self.hunger = 0.0
    self.extra_activities = []
    
  def big_round(self):
    Character.big_round(self)
    
    if not self.qualities.has_key("ascetic"):
      if self.hunger > 1.0: self.hurt(0.02, None, 0, 0)
      elif self.qualities.has_key("greedy"): self.hunger += 0.002
      else:                                  self.hunger += 0.001
      
  def hurt(self, life, caster, animate = 1, can_resist = 1):
    if self.queue.local and caster:
      self.queue.controler.change_relation(caster, -life * 4.0)
    Character.hurt(self, life, caster, animate, can_resist)
    
    
class Farmer(DretzelCandidate):
  die_duration1  = 30
  die_duration2  = 150
  nb_fight_round = 90
  nb_fight_round = 150
  fight_effects  = {
    20  : ( 0.1, 2.2, -1.5),
    98  : ( 0.1, 1.7, -1.6),
    }
  
  activities = [
    ]
  for i in range(len(activities)): activities[i].id = i - 100
  
  def __init__(self, player = 0):
    self.fighting   = 2.0
    self.resistance = 3.0
    
    DretzelCandidate.__init__(self, player)
    
    self.perso.set_shape(soya.Cal3dShape.get("paysan"))
    self.perso.attach("perso")
    #self.visible = 0
    
    self.init_kill_rays()


class Hero(Character):
  discussion_radius = 5.0
  
  def __init__(self, player = 1):
    Character.__init__(self, player)
    self.spc_data = {}
    self.item_listeners = []

  def grab_item(self, item):
    Character.grab_item(self, item)
    for listener in self.item_listeners[:]: listener(item)
    
  def get_spc(self, spc, default = 0): return self.spc_data.get(spc, default)
  def set_spc(self, spc, value   = 1): self.spc_data[spc] = value
  
  def big_round(self):
    Character.big_round(self)
    
    discutables = [(self.distance_to(i), i)
                   for i in self.level
                   if isinstance(i, base.Discutable)
                   ]
    if discutables: distance, nearest_discutable = min(discutables)
    else:           distance, nearest_discutable = 10000.0, None
    
    if discussion.CURRENT_DISCUSSION:
      if ((not discussion.CURRENT_DISCUSSION.character is self) and (not discussion.CURRENT_DISCUSSION.character is nearest_discutable)) or not discussion.CURRENT_DISCUSSION.check():
        discussion.CURRENT_DISCUSSION.destroy()
        
    if (not discussion.CURRENT_DISCUSSION) and (distance < nearest_discutable.discussion_radius): nearest_discutable.start_discussion(self)

    
        
class Balazar(Hero):
  sort_duration1 = [32, 56]
  sort_duration2 = [44, 52]
  die_duration1  = 84
  die_duration2  = 108
  nb_fight_round = 132
  fight_effects  = {
    15  : (-0.1, 1.9, -1.3),
    80  : ( 0.2, 2.1, -1.4),
    }
  
  activities = [
    activity.ResurrectShield(0.0),
    activity.Lightening(0.1),
    activity.Heal(0.1),
    activity.Attack(),
    activity.DontAttack(),
    activity.Kesako(),
    ]
  for i in range(len(activities)): activities[i].id = i - 100
  
  def __init__(self, player = 1):
    self.extra_activities = [
      activity.LightCitadel(0.75),
      activity.Invisibility(0.1),
      activity.Levitate(0.075),
      activity.Telekinesy(0.1),
      activity.Uncurse(0.2),
      activity.Boreal(0.1),
      ]
    for i in range(len(self.extra_activities)): self.extra_activities[i].id = i
    
    Hero.__init__(self, player)
    
    self.perso.set_shape(soya.Cal3dShape.get("balazar"), ["perso"])
    #self.perso.set_shape(soya.Cal3dShape.get("beuglaz"), ["perso"])
    #self.perso.set_shape(soya.Cal3dShape.get("dodd"))
    
    #self.perso.shape.set_cellshading(outline_width = 0)
    #self.perso.shape.shadow = 1
    #self.perso.shape.double_sided = 1
    
    self.mana       = 1.0
    self.fighting   = 2.0
    self.resistance = 3.0
    self.alignment  = 0.2
    self.tolerance  = 0.5
    
    
    self.right_foot = soya.World(self)
    self.perso.attach_to_bone(self.right_foot, "piedD")
    #self.spc_kill_ray = KillRay(self.right_foot, 0.0, 0.1, -0.2, 0.5, 0.0, 0.0)
    self.spc_kill_rays = [
      KillRay(self.right_foot, -1.0, 0.1, -0.2, 2.0, 0.0,  0.0),
      KillRay(self.right_foot,  0.0, 0.1,  0.2, 0.0, 0.0, -1.5),
      ]
    
    self.init_kill_rays()
    
    import balazar.item
    
    #i = balazar.item.Key()
    #self.grab_item(i)
    
    #i = balazar.item.MushroomFood()
    #self.grab_item(i)
    
    i = balazar.item.Knife()
    self.grab_item(i)
    
    i = balazar.item.TuryleSword()
    #i = balazar.item.Axe()
    #i = balazar.item.CursedSword()
    self.grab_item(i)
    i.set_equiped(1)
    
    i = balazar.item.Camera()
    #i.id = -1000
    self.grab_item(i)
    i.set_equiped(1)
    
  def set_level(self, level):
    Character.set_level(self, level)
    self.initial_pos = self.x, self.y, self.z
    
  def hurt(self, life, caster, localisation = None, vector = None, can_resist = 1):
    sound.play("couic1.wav", self)
    Character.hurt(self, life, caster, localisation, vector, can_resist)
    
  def kill(self, caster):
    sound.play("couic2.wav", self)
    if self.queue.local:
      self.queue.controler.animate(self.activities[0].cast(self, self))
      #self.queue.controler.animate(BalazarDie(self))
      
  def strike_right(self, target = None):
    if not self.weapon: return
    for i in range(8):
      if target:
        _P.clone(target)
        _P.convert_to(self)
        if _P.x > 0.0: rot = -4.0
        else: rot = 1.0
      else: rot = -4.0
      s = State(self, rotation_speed = rot, animation = "combat0")
      s.unfinished = 1
      yield s
    sound.play("strike2.wav", self.weapon, gain = 4.0)
    for i in range(13):
      if target:
        _P.clone(target)
        _P.convert_to(self)
        if _P.x > 0.0: rot = -4.0
        else: rot = 1.0
      else: rot = -4.0
      s = FightState(self, rotation_speed = rot, animation = "combat0")
      s.unfinished = 1
      yield s
      
  def strike_left(self, target = None):
    if not self.weapon: return
    for i in range(8):
      if target:
        _P.clone(target)
        _P.convert_to(self)
        if _P.x < 0.0: rot = 4.0
        else: rot = 1.0
      else: rot = 4.0
      s = State(self, rotation_speed = rot, animation = "combat1")
      s.unfinished = 1
      yield s
    sound.play("strike2.wav", self.weapon, gain = 4.0)
    for i in range(13):
      if target:
        _P.clone(target)
        _P.convert_to(self)
        if _P.x < 0.0: rot = 4.0
        else: rot = 1.0
      else: rot = 4.0
      s = FightState(self, rotation_speed = rot, animation = "combat1")
      s.unfinished = 1
      yield s

      
  def strike_sagittal(self, target = None):
    if not self.weapon: return
    for i in range(8):
      s = State(self, rotation_speed = self.get_strike_rot(target), animation = "combat3")
      s.unfinished = 1
      yield s
    sound.play("strike3.wav", self.weapon, gain = 4.0)
    for i in range(13):
      s = FightState(self, rotation_speed = self.get_strike_rot(target), animation = "combat3")
      s.unfinished = 1
      yield s
      
  def strike_charge(self, target = None):
    if not self.weapon: return
    for i in range(3):
      s = State(self, animation = "combat2")
      s.unfinished = 1
      yield s
    for i in range(7):
      s = FightState(self, rotation_speed = self.get_strike_rot(target), animation = "combat2", kill_rays = self.spc_kill_rays)
      s.unfinished = 1
      yield s
    sound.play("strike1.wav", self, gain = 2.0)
    for i in range(4):
      s = FightState(self, rotation_speed = self.get_strike_rot(target), z = -0.2, animation = "combat2", kill_rays = self.spc_kill_rays)
      s.unfinished = 1
      yield s
    s = FightState(self, rotation_speed = self.get_strike_rot(target), z = -0.25, y = 0.16, animation = "combat2", kill_rays = self.spc_kill_rays)
    s.unfinished = 1
    yield s
    for i in range(15):
      s = FightState(self, rotation_speed = self.get_strike_rot(target), z = -0.3, animation = "combat2", kill_rays = self.spc_kill_rays)
      s.unfinished = 1
      yield s
    for i in range(6):
      s = FightState(self, rotation_speed = self.get_strike_rot(target), z = -0.05, animation = "combat2", kill_rays = self.spc_kill_rays)
      s.unfinished = 1
      yield s
      
      
class Beuglaz(Hero):
  alignment  = -2
  move_speed =  0.8
  
  sort_duration1 = [0, 52]
  sort_duration2 = [0, 58]
  die_duration1  = 114
  die_duration2  = 102
  nb_fight_round = 1
  
  activities = [
    activity.Goto(),
    activity.Attack(),
    activity.DontAttack(),
    activity.Kesako(),
    ]
  for i in range(len(activities)): activities[i].id = i - 100
  
  def __init__(self, player = 1):
    self.extra_activities = [
      activity.Sacrifice(-0.1),
      activity.Curse(0.15),
      ]
    for i in range(len(self.extra_activities)): self.extra_activities[i].id = i
    
    Character.__init__(self, player)
    
    self.perso.set_shape(soya.Cal3dShape.get("beuglaz"))
    self.perso.attach("perso")
    
    self.mana       = 1.0
    self.fighting   = 1.0
    self.resistance = 2.0
    
    self.init_kill_rays()
    
    import balazar.item
    #i = balazar.item.Knife()
    i = balazar.item.TuryleSword()
    #i = balazar.item.CursedSword()
    i.id = -1000
    self.grab_item(i)
    
  def strike(self):
    for i in range(33):
      s = FightState(self, animation = "combat0")
      s.unfinished = 1
      yield s
#     for i in range(10):
#       s = State(self, animation = "combat0")
#       s.unfinished = 1
#       yield s
#     yield State(self, x = -1008.0)
#     for i in range(22):
#       s = State(self, animation = "combat0")
#       s.unfinished = 1
#       yield s
    self.fight_mode += 30
    
  def strike_fast(self):
    for i in range(21):
      s = FightState(self, animation = "combat1")
      s.unfinished = 1
      yield s
#     for i in range(5):
#       s = State(self, animation = "combat1")
#       s.unfinished = 1
#       yield s
#     yield State(self, x = -1008.0)
#     for i in range(15):
#       s = State(self, animation = "combat1")
#       s.unfinished = 1
#       yield s
    self.fight_mode = 0
    
  strike_big  = strike
    
  def hurt(self, life, caster, animate = 1, can_resist = 1):
    if (not caster) or (not caster.is_dretzel): life = 0.0
    Character.hurt(self, life, caster, animate, can_resist)
        
    
