# Balazar
# Copyright (C) 2003 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import soya
from soya     import Point, Vector
from soya.ray import Ray

import balazar.base as base

from weakref import ref


class KillPoint(Point):
  radius = 1.3
  
class KillRay(Point):
  def __init__(self, parent = None, x = 0.0, y = 0.0, z = 0.0, dir_x = 0.0, dir_y = 0.0, dir_z = -1.0):
    Point.__init__(self, parent, x, y, z)
    self.dir    = Vector(parent, dir_x, dir_y, dir_z)
    self.length = self.dir.length()
    
    #soya.Face(parent, [
    #  soya.Vertex(parent, x, y, z),
    #  soya.Vertex(parent, x + dir_x, y + dir_y, z + dir_z),
    #  ])
    
    
ITEMS = []

def set_items_solid(solid):
  def set_item_solid(item_ref):
    item = item_ref()
    if item:
      item.solid = solid
      return 1
    else: return 0
    
  global ITEMS
  ITEMS = filter(set_item_solid, ITEMS)
  
class Item(soya.World, base.Entity, base.Discutable):
  blessed = 0
  can_rotate = 1
  
  discussion_radius = 2.0
  
  def __deepcopy__(self, memo):
    memo[id(self.owner)] = None
    return soya.World.__deepcopy__(self, memo)
  
  def __init__(self, parent):
    soya.World.__init__(self, parent)
    
    #self.solid = 0
    ITEMS.append(ref(self))
    
    self.owner = None

  def start_discussion(self, hero):
    from balazar.discussion import Discussion
    from balazar.controler  import GrabItem
    Discussion(hero, u"", _(u"Objet au sol : %s") % base.display_name(self),
               check  = lambda : hero.distance_to(self) < 2.0,
               choices = [
      Discussion(hero, _(u"Ramasser"),
                 action = lambda : hero.queue.controler.append_and_cancel(GrabItem(hero, self)),
                 ),
      ]).activate()
    
  def kesako(self): return _("__kesako__%s" % self.__class__.__name__)
  def incompatible_with(self, other): return 0
  
  def set_owner(self, character):
    self.owner = character
    
  def fighting_bonus  (self, target): return 0.0
  def resistance_bonus(self, target): return 0.0
  
  def zap(self): pass
  
  def set_inventory_pos(self):
    """Set the item in the right position / orientation, when it is displayed
in the inventory"""
    self.set_identity()
    self.rotate_vertical(90.0)
    self.rotate_lateral (90.0)
    min, max = self.get_box()
    self.y = -(min.y + max.y) / 2.0
    
    
class UseableItem(Item):
  def used_by(self, user):
    pass
  
  
class EquipableItem(Item):
  def __init__(self, parent):
    Item.__init__(self, parent)
    self.equiped = 0
    
  def set_equiped(self, equiped):
    if equiped:
      for item in self.owner.items:
        if isinstance(item, EquipableItem) and item.equiped and item.incompatible_with(self): item.set_equiped(0)
    self.equiped = equiped
    
  def set_owner(self, character):
    if self.equiped: self.set_equiped(0)
    self.owner = character
    
    
class RightHandItem(EquipableItem):
  def incompatible_with(self, other): return isinstance(other, RightHandItem)
  
  def set_equiped(self, equiped):
    EquipableItem.set_equiped(self, equiped)
    
    if equiped:
      #character.right_hand = soya.World(character)
      #character.perso.attach_to_bone(character.right_hand, "mainD")
      self.owner.right_hand.add(self)
      self.set_identity()
      self.set_equiped_pos()
      self.zap()
    else:
      self.parent.remove(self)
      
  def set_equiped_pos(self):
    self.rotate_incline(180.0)
    self.set_xyz(0.05, 0.1, 0.0)
    
  #def unequip(self):
  #  self.owner.perso.detach_from_bone(self.owner.right_hand)
  #  self.owner.remove(self.owner.right_hand)
  #  
  #  Item.unequip(self)
    
    
class LeftHandItem(EquipableItem):
  def incompatible_with(self, other): return isinstance(other, LeftHandItem)
  
  def set_equiped(self, equiped):
    EquipableItem.set_equiped(self, equiped)
    
    if equiped:
      self.owner.left_hand_item = self
      self.owner.left_hand = soya.World(self.owner)
      self.owner.perso.attach_to_bone(self.owner.left_hand, "mainG")
      self.owner.left_hand.add(self)
      
      self.set_identity()
      self.set_equiped_pos()
      self.zap()
    else:
      self.owner.left_hand_item = None
      self.parent.remove(self)
      self.owner.perso.detach_from_bone(self.owner.left_hand)
      self.owner.remove(self.owner.left_hand)
      
  def set_equiped_pos(self):
    self.set_xyz(0.0, 0.1, 0.0)
    
    
class Weapon(RightHandItem):
  range = 1.0 # Used fo AI
  def set_equiped(self, equiped):
    RightHandItem.set_equiped(self, equiped)
    if equiped: self.owner.weapon = self
    else:       self.owner.weapon = None

class Camera(LeftHandItem):
  def __init__(self, parent = None):
    LeftHandItem.__init__(self, parent)
    self.set_shape(soya.Shape.get("camera"))
    
  def set_equiped_pos(self):
    self.rotate_lateral(25.0)
    self.set_xyz(-0.05, 0.34, 0.0)
    

photo_model = soya.World()
soya.Face(photo_model, [
  soya.Vertex(photo_model, -0.5333,  0.36, 0.0, 0.005, 0.005),
  soya.Vertex(photo_model,  0.5333,  0.36, 0.0, 0.995, 0.005),
  soya.Vertex(photo_model,  0.5333, -0.36, 0.0, 0.995, 0.995),
  soya.Vertex(photo_model, -0.5333, -0.36, 0.0, 0.005, 0.995),
  ]).double_sided = 1


class Photo(UseableItem):
  def __init__(self, parent = None, image = None):
    UseableItem.__init__(self, parent)
    self.image = image
    
    photo_model.children[0].material = material = soya.Material(soya.image_from_pil(image.resize((64, 64))))
    material.clamp = 1
    self.shape = photo_model.shapify()
    photo_model.children[0].material = soya.DEFAULT_MATERIAL
    
    self.life = 250
    
  def __deepcopy__(self, memo):
    memo[id(self.image)] = self.image
    memo[id(self.shape)] = self.shape
    return UseableItem.__deepcopy__(self, memo)
  
  def set_owner(self, character):
    UseableItem.set_owner(self, character)
    if not character: self.life = 250
    
  def begin_round(self):
    self.life -= 1
    if self.life == 0: soya.IDLER.next_round_tasks.append(self.destroy)
    
  def destroy(self):
    if self.parent:
      self.parent.remove(self)
      
  def set_inventory_pos(self):
    self.set_identity()
    
  def used_by(self, user):
    if user.queue.local and user.player:
      import balazar.controler as controler
      soya.root_widget.status_bar.set_widget(controler.PhotoViewer(self), soya.get_screen_height() / 1.1)
      
  def display_name(self):
    if len(self.content) < 3: content = self.content
    else:                     content = self.content[:2] + ["..."]
    return _("__item__%s" % self.__class__.__name__) + u" (%s : %s)" % (self.score, ", ".join(content))
  
  def full_display_name(self):
    return _("__item__%s" % self.__class__.__name__) + u" (%s : %s)" % (self.score, ", ".join(self.content))
  
  def kesako(self):
    return _("__kesako__%s" % self.__class__.__name__) + u"\n%s" % ", ".join(self.content)
  
class TuryleSword(Weapon):
  blessed = 2
  range   = 2.0
  
  def __init__(self, parent = None):
    Weapon.__init__(self, parent)
    self.set_shape(soya.Shape.get("epee_turyle"))
    
    self.ray = Ray(self, soya.Material.get("epee_turyle_rayon"))
    self.ray.z = -0.16
    self.ray.endpoint.z = -1.8
    self.ray.length = 5
    
    self.kill_rays = [
      KillRay(self, 0.0,  0.2, 0.0, 0.0, 0.0, -2.0),
      KillRay(self, 0.0, -0.2, 0.0, 0.0, 0.0, -2.0),
      ]
    
    self.prot = soya.Volume(self, soya.Shape.get("epee_turyle_prot"))
    self.prot.visible = self.prot.solid = 0
    
#     import soya.sphere as sphere
#     m = soya.Material()
#     m.diffuse = 1.0, 0.0, 0.0, 0.5
#     m.
#     w = sphere.Sphere(None, m)
#     w.scale(1.3, 1.3, 1.3)
#     soya.Volume(self, w.shapify()).move(self.kill_point)

    # Halo (ideal pour un baton magique)
#     import soya.sphere as sphere
#     m = soya.Material()
#     m.diffuse = 0.15, 0.1, 0.2, 1.0
#     m.diffuse = 0.5, 0.4, 0.8, 1.0
#     m.additive_blending = 1
#     w = sphere.Sphere(None, m)
#     w.scale(1.3, 1.3, 1.3)
#     soya.Volume(self, w.shapify()).move(self.kill_point)
    
  def fighting_bonus(self, target):
    if self.equiped: return 4.0
    return 0.0
  
  def zap(self): self.ray.zap()
    
    
class Axe(Weapon):
  blessed = 2
  range   = 1.7
  
  def __init__(self, parent = None):
    Weapon.__init__(self, parent)
    self.set_shape(soya.Shape.get("hache"))
    
    self.ray = Ray(self, soya.Material.get("couteau_rayon"))
    self.ray.         set_xyz(0.0, -0.5, -0.4)
    self.ray.endpoint.set_xyz(0.0,  0.0, -1.1)
    self.ray.length = 5
    
    self.kill_rays = [
      KillRay(self, 0.0,   0.15,  0.0,  0.0,  0.0, -1.5),
      #KillRay(self, 0.0,  0.2, -1.2,  0.0, -1.0,  0.0),
      KillRay(self, 0.0,  -0.15,  0.0,  0.0, -0.6,  -1.2),
      ]
    
    self.prot = soya.Volume(self, soya.Shape.get("hache_prot"))
    self.prot.visible = self.prot.solid = 0
    
  def fighting_bonus(self, target):
    if self.equiped: return 5.0
    return 0.0
  
  def zap(self): self.ray.zap()
    
    
class Knife(Weapon):
  range = 1.3
  def __init__(self, parent = None):
    Weapon.__init__(self, parent)
    self.set_shape(soya.Shape.get("couteau"))
    
    self.ray = Ray(self, soya.Material.get("couteau_rayon"))
    self.ray.z = -0.16
    self.ray.endpoint.z = -0.85
    self.ray.length = 5
    
    #self.kill_point = KillPoint(self, 0.0, 0.0, -0.7)
    self.kill_rays = [
      KillRay(self, 0.0,  0.15, 0.2, 0.0, 0.0, -1.4),
      KillRay(self, 0.0, -0.15, 0.2, 0.0, 0.0, -1.4),
      ]
    
    self.prot = soya.Volume(self, soya.Shape.get("couteau_prot"))
    self.prot.visible = self.prot.solid = 0
    
  def fighting_bonus(self, target):
    if self.equiped: return 1.0
    return 0.0
  
  def zap(self):
    self.ray.zap()
    
    
class Key(Item):
  def __init__(self, parent = None):
    Item.__init__(self, parent)
    self.set_shape(soya.Shape.get("clef"))
    
    
class LeftHandHorizontalItem(LeftHandItem):
  def equip(self, character):
    Item.equip(self, character)
    
    character.left_hand = soya.World(character)
    character.perso.attach_to_bone(character.left_hand, "mainG")
    character.add(self)
    self.set_identity()
    self.rotate_incline(180.0)
    self.set_xyz(-0.05, 0.1, 0.0)
    
  def unequip(self):
    self.owner.perso.detach_from_bone(self.owner.left_hand)
    self.owner.remove(self.owner.left_hand)
    self.owner.remove(self)
    
    Item.unequip(self)
    
  def advance_time(self, proportion):
    if self.owner:
      xyz = self.owner.left_hand.x, self.owner.left_hand.y, self.owner.left_hand.z
      self.set_identity()
      self.set_xyz(*xyz)
      
  def set_inventory_pos(self):
    """Set the item in the right position / orientation, when it is displayed
in the inventory"""
    self.set_identity()
    min, max = self.get_box()
    self.y = -(min.y + max.y) / 2.0
    
    
class Food(UseableItem):
  nutritive_value = 1.0
  can_rotate = 0
  
  def eaten_by(self, user):
    user.live = min(1.0, user.life + self.nutritive_value / 10.0)
    from balazar.hero import DretzelCandidate, Balazar
    if   isinstance(user, DretzelCandidate): user.hunger = max(0.0, user.hunger - self.nutritive_value)
    elif isinstance(user, Balazar         ): user.mana   = min(1.0, user.mana   + self.nutritive_value / 20.0)
    
    if self.owner: self.owner.drop_item(self)
    self.parent.remove(self)
    
  def set_inventory_pos(self):
    self.set_identity()
    min, max = self.get_box()
    self.y = -(min.y + max.y) / 2.0
    
    
class MushroomFood(Food):
  def __init__(self, parent = None):
    Food.__init__(self, parent)
    #self.set_shape(soya.Shape.get("nourriture_champignon"))
    self.volume = soya.Volume(self, soya.Shape.get("nourriture_champignon"))
    
class OldMushroomFood(Food):
  nutritive_value = 0.3
  def __init__(self, parent = None):
    Food.__init__(self, parent)
    self.volume = soya.Volume(self, soya.Shape.get("nourriture_champignon_blet"))
    
    
class TelekinesyCursedItem(Item):
  blessed = -1
  
  def __init__(self, parent = None):
    super(TelekinesyCursedItem, self).__init__(parent)
    self.cursed_owner = None
    
    self.kill_point = KillPoint(self, 0.0, 0.0, -1.5)
    
  def set_owner(self, character):
    if self.owner and (self.owner.life <= 0.0):
      # Free the curse if the previous owner is dead
      self.cursed_owner = None
    super(TelekinesyCursedItem, self).set_owner(character)
    
    if not self.cursed_owner: self.cursed_owner = character
    elif character and (not self.cursed_owner is character):
      # Cursed items cannot be given to someone else
      character.drop_item(self)
      
    if self.owner: self.set_equiped(1)
    
  def set_equiped(self, equiped):
    super(TelekinesyCursedItem, self).set_equiped(equiped)
    if (not equiped) and (self.owner.life > 0.0): self.owner.drop_item(self) # => will activate the curse !
    
  def big_round(self):
    if self.cursed_owner and (not self.owner):
      if self.distance_to(self.cursed_owner) > 1.0:
        from balazar.activity import TelekinesyEffect
        TelekinesyEffect(self.parent, self.cursed_owner, self).move(self)
        self.cursed_owner = None
        
  def reset_curse(self): self.cursed_owner = None
  
  
class CursedSword(TelekinesyCursedItem, Weapon):
  range = 1.6
  
  def __init__(self, parent = None):
    TelekinesyCursedItem.__init__(self, parent)
    self.set_shape(soya.Shape.get("epee_maudite"))
    
    self.ray = Ray(self, soya.Material.get("epee_maudite_rayon"))
    self.ray.z = -0.16
    self.ray.endpoint.set_xyz(0.0, 0.27, -1.38)
    self.ray.length = 5
    
    self.kill_rays = [
      KillRay(self, 0.0,  0.15, 0.0, 0.0, 0.2, -1.6),
      KillRay(self, 0.0, -0.15, 0.0, 0.0, 0.2, -1.6),
      ]
    
    self.prot = soya.Volume(self, soya.Shape.get("epee_maudite_prot"))
    self.prot.visible = self.prot.solid = 0
    
  def fighting_bonus(self, target):
    if self.equiped: return -1.0
    return 0.0
  
  def zap(self): self.ray.zap()
  

# Tables for random item choice

SMALL_TREASURE = [
  MushroomFood,
  MushroomFood,
  Key,
  Key,
  Knife,
  Knife,
  TuryleSword,
  CursedSword,
  ]

BIG_TREASURE = [
  Key,
  TuryleSword,
  CursedSword,
  ]


def _compute_names(clazz):
  for subclass in clazz.__subclasses__(): _compute_names(subclass)
  if not hasattr(clazz, "display_name"):
    clazz.display_name = _("__item__%s" % clazz.__name__)
_compute_names(Item)

