# Balazar
# Copyright (C) 2004 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import soya
from soya import Point, Vector

from balazar.controler  import StackControler, Goto, GotoXZ, Gotoward, LookAt, GrabItem, Fly, Wait, Wander
from balazar.character  import State, FightState, Character, _P, _V
from balazar.item       import Item, Food, Key, Axe
from balazar.discussion import Discussion

import balazar.base  as base
import balazar.sound as sound

def WaitDuringDiscussion(self):
  import balazar.discussion as discussion
  while discussion.CURRENT_DISCUSSION and (discussion.CURRENT_DISCUSSION.character is self): yield State(self)
  
class NPC(Character, base.Discutable):
  pass
  

class Morkul(NPC):
  alignment  = -0.25
  tolerance  =  0.5
  
  move_speed =  0.8
  
  sort_duration1 = [0, 52]
  sort_duration2 = [0, 58]
  die_duration1  = 114
  die_duration2  = 102
  nb_fight_round = 1
  
  def __init__(self, player = 1):
    NPC.__init__(self, player)
    
    #self.perso.set_shape(soya.Cal3dShape.get("beuglaz"))
    self.perso.set_shape(soya.Cal3dShape.get("beuglaz"), ["perso"])
    #self.perso.shape.set_cellshading()
    
    self.fighting   = 2.0
    self.resistance = 2.0
    
    self.init_kill_rays()
    
    import balazar.item
    i = balazar.item.Knife()
    #i = balazar.item.TuryleSword()
    #i = balazar.item.CursedSword()
    self.grab_item(i)
    i.set_equiped(1)
    
  def start_discussion(self, hero):
    if not self.is_enemy(hero):
      MorkulDiscussion(self).activate()
      print self.queue.controler.uncancelable_controler
      self.queue.controler.animate(WaitDuringDiscussion(self))
      
  def hurt(self, life, caster, localisation = None, vector = None, can_resist = 1):
    sound.play("couic_morkul.wav", self)
    Character.hurt(self, life, caster, localisation, vector, can_resist)
    
  def kill(self, caster):
    sound.play("grogne2.wav", self)
    Character.kill(self, caster)
    
  def strike_sagittal(self, enemy = None):
    for i in range(10):
      s = State(self, rotation_speed = self.get_strike_rot(enemy), animation = "combat3")
      s.unfinished = 1
      yield s
    sound.play("strike2.wav", self.weapon, gain = 4.0)
    for i in range(20):
      s = FightState(self, rotation_speed = self.get_strike_rot(enemy), animation = "combat3")
      s.unfinished = 1
      yield s
      
  def strike_fast(self, enemy = None):
    for i in range(5):
      s = State(self, animation = "combat1")
      s.unfinished = 1
      yield s
    for i in range(16):
      s = FightState(self, animation = "combat1")
      s.unfinished = 1
      yield s
      
  def strike_left(self, target = None):
    for i in range(10):
      s = State(self, animation = "combat1")
      s.unfinished = 1
      yield s
    sound.play("strike2.wav", self.weapon, gain = 4.0)
    for i in range(17):
      s = FightState(self, rotation_speed = 4.0, animation = "combat1")
      s.unfinished = 1
      yield s
      
  def strike_right(self, target = None):
    for i in range(10):
      s = State(self, animation = "combat0")
      s.unfinished = 1
      yield s
    sound.play("strike2.wav", self.weapon, gain = 4.0)
    for i in range(17):
      s = FightState(self, rotation_speed = -4.0, animation = "combat0")
      s.unfinished = 1
      yield s
      
  def strike_charge(self, target = None): # Not implemented
    for i in range(25):
      yield State(self, 0.0, 0.0, -0.25 * self.move_speed,  0.0, "marche")
      
      
class AI(StackControler):
  vision               = 50.0
  aggressivity         = 0.0
  dodge                = 0.05
  dodge_while_striking = 0
  
  def __init__(self, character = None, house = None):
    StackControler.__init__(self, character)
    
    self.enemy = None
    self.house = house
    #self.check_time = self.check_period
    #self.check()
    
  def __setstate__(self, state):
    self.__dict__ = state
    #self.check()
    
  def item_gained(self, item):
    if isinstance(item, Food):
      if self.character.hunger > 0.2: # Eat
        item.eaten_by(self.character)
        
        
  def animate_and_cancel(self, controler):
    StackControler.animate_and_cancel(self, controler)
    self.check_time = 0
    
  def append_and_cancel(self, controler, time_out = 400):
    StackControler.append_and_cancel(self, controler)
    self.check_time = time_out
    
    
  def next(self):
    if self.enemy and (not self.uncancelable_controler) and (self.dodge_while_striking or (not self.character.current_animation.startswith("combat"))):
      if self.character.level.random.random() < self.dodge:
        if self.enemy.current_animation and self.enemy.current_animation.startswith("combat") and (self.character.distance_to(self.enemy) < 3.0):
          self.animate(self.attack_dodge())
          
    if not self.controlers: self.check()
    else:
      self.check_time -= 1
      if (not self.uncancelable_controler) and (self.check_time <= 0):
        # Cancel current action, because it is taking too much time.
        self.check()
        
    state = StackControler.next(self)
    state = self.character.collision_detection(state or State(self.character))
    return state
  
  def check(self):
    self.check_time = 400
    self.enemy      = None
    
    interests = map(lambda o: (self.interest(o), o), self.character.level)
    
    if interests:
      a = max(interests)
      b = min(interests)
      
      if a[0] > -b[0]: o = a
      else:            o = b
      if abs(o[0]) > 0.2:
        self.append_and_cancel(self.deal_with(*o))
        return

    if self.house:
      if self.character.distance_to(self.house) > 100.0 * self.character.level.random.random():
        self.append_and_cancel(Goto(self.character, self.house, 5.0))
        return
      
    # Nothing of interest
    if self.character.level.random.random() < 0.6: self.append_and_cancel(Wait  (self.character, 200))
    else:                                          self.append_and_cancel(Wander(self.character, 200))
    
  def relation_changed(self, character):
    self.check_time = 2
    
  def interest(self, o, d = None):
    """Returns a value in the range [-1.0, 1.0] indicating if the AI is interested
by O (where 1.0 means very interested, 0.0 means ignore it, and -1.0 means go away
from it)."""
    if o is self.character: return 0.0
    if d is None: d = 1.0 + self.character.distance_to(o) / self.vision
    
    if isinstance(o, Item):
      incomp = 0.0
      if o.owner:
        if not o.owner is self.character:
          # XXX Steal ?
          return 0.0
      else:
        for item in self.character.items:
          if item.incompatible_with(o): incomp += self.interest(item)
      return (0.1 + max(0.0, o.blessed) - incomp) / 3.0 / d
    if isinstance(o, Character):
      if o.life <= 0.0: return 0.0
      return min(0.0, self.character.get_relation(o) / d)
    
    return 0.0 # Unidentifiable or useless thing
  
  def deal_with(self, interest, o):
    print "deal_with", interest, o
    if interest < 0.0:
      if isinstance(o, Character): yield self.deal_with_character(o)
      else:                        yield Fly     (self.character, o)
    elif isinstance(o, Item):      yield GrabItem(self.character, o)
    else:                          yield Goto    (self.character, o)
    
  def deal_with_character(self, enemy):
    self.enemy = enemy
    while enemy.life > 0.0:
      a = self.aggressivity + self.character.life - enemy.life
      
      if a > 0.0:
        yield self.attack(enemy)
        #yield self.attack_jump(enemy)
        #yield self.attack_jump_over2(enemy)
        #yield self.attack_back (enemy)
        #yield self.attack_side (enemy)
        #yield self.attack_basic(enemy)
      else:
        yield Fly(self.character, enemy)
        break
      
  def attack_dodge(self):
    yield State(self.character, 0.0, 0.3, 0.1)
    for i in range(10):
      yield State(self.character, 0.0, 0.0, 0.1)
    yield State(self.character, 0.0, 0.0, 0.05)
    
  def attack_basic(self, enemy):
    yield Goto(self.character, enemy, self.character.range + ((self.character.weapon and self.character.weapon.range) or 0.0))
    #yield LookAt(self.character, enemy, 3.0)
    
    _P.clone(enemy)
    _P.convert_to(self.character)
    if -1.0 < _P.y < 1.0:
      if _P.x < 0.0: yield self.character.strike_left (enemy)
      else:          yield self.character.strike_right(enemy)
    else:
      yield self.character.strike_sagittal(enemy)
      
  def attack_side(self, enemy):
    _P.clone(self.character)
    _P.convert_to(enemy)
    if _P.x < 0.0: x = -6.0
    else:          x =  6.0
    yield GotoXZ(self.character, Point(enemy, x, 0.0, 2.0), 1.5)
    
    self.attack_basic(enemy)
    
  def attack_back(self, enemy):
    _P.clone(self.character)
    _P.convert_to(enemy)
    if _P.z < abs(_P.x):
      if _P.x < 0.0: x = -6.0
      else:          x =  6.0
      yield GotoXZ(self.character, Point(enemy, x, 0.0, 2.0), 1.5)
      
    yield GotoXZ(self.character, Point(enemy, 0.0, 0.0, 3.0), 1.5)
    
    yield self.attack_basic(enemy)
    
  def attack_jump_over(self, enemy):
    yield Goto(self.character, enemy, 5.0)
    yield LookAt(self.character, enemy)
    
    yield State(self.character, y = 0.4, z = -0.25 * self.character.move_speed)
    
    yield GotoXZ(self.character, Point(enemy, 0.0, 0.0, 3.0), 3.0)
    yield State(self.character, y = -0.1)
    yield GotoXZ(self.character, Point(enemy, 0.0, 0.0, 3.0), 1.0)
    
    yield LookAt(self.character, enemy, 3.0)
    
    yield self.attack_basic(enemy)
    
  def attack_jump_over2(self, enemy):
    yield Goto(self.character, enemy, 5.0)
    yield LookAt(self.character, enemy)
    
    yield State(self.character, y = 0.4, rotation_speed = 12.0)
    for i in range(14):
      yield State(self.character, rotation_speed = 12.0)
    yield State(self.character, y = -0.1)
    for i in range(25):
      if self.character.on_ground: break
      yield State(self.character, z = 0.25)
      
    yield self.attack_basic(enemy)
    
  def attack_jump_over3(self, enemy):
    yield Goto(self.character, enemy, 5.0)
    yield LookAt(self.character, enemy)
    
    yield State(self.character, y = 0.4, rotation_speed = 12.0)
    angle = 12.0 / 180.0 * 3.1416
    import math
    for i in range(14):
      if i == 7: y = -0.1
      else:      y =  0.0
      yield State(self.character, x = 0.25 * self.character.move_speed * math.sin(angle), y = y, z = -0.25 * self.character.move_speed * math.cos(angle), rotation_speed = 12.0)
      angle += 12.0 / 180.0 * 3.1416
      
    for i in range(15):
      if self.character.on_ground: break
      yield State(self.character)
      
    yield self.attack_basic(enemy)
    
  def attack_jump(self, enemy):
    yield LookAt(self.character, enemy, 2.0)
    while self.character.distance_to(enemy) < 3.0:
      yield State(self.character, z = 0.2 * self.character.move_speed, animation = "recule")
   
    yield State(self.character, y = 0.3)

    yield self.character.strike_sagittal(enemy)
    #yield GotoXZ(self.character, enemy, 2.0)
    yield self.attack_basic(enemy)
    
    
    
class MorkulAI(AI):
  aggressivity  = 0.8
  dodge         = 0.03
  
  def __init__(self, character, house = None):
    AI.__init__(self, character, house)
    
  def attack(self, enemy):
    h = self.character.level.random.random()
    if   h < 0.6: yield self.attack_basic(enemy)
    elif h < 0.8: yield self.attack_jump (enemy)
    else:         yield self.attack_side (enemy)
  

class MorkulChief(Morkul):
  alignment  = -0.25
  
  move_speed =  0.7
  
  sort_duration1 = [0, 52]
  sort_duration2 = [0, 58]
  die_duration1  = 114
  die_duration2  = 102
  nb_fight_round = 1
  
  def __init__(self, player = 1):
    NPC.__init__(self, player)
    
    self.perso.set_shape(soya.Cal3dShape.get("chef_morkul"), ["perso"])
    
    self.fighting   = 2.0
    self.resistance = 3.0
    
    self.init_kill_rays()
    
    import balazar.item
    #i = balazar.item.Knife()
    i = balazar.item.Axe()
    #i = balazar.item.CursedSword()
    self.grab_item(i)
    i.set_equiped(1)
    
    
class Morkar(MorkulChief):
  pass

class Morkul1(Morkul):
  pass

class Morkul2(Morkul):
  def __init__(self, player = 1):
    Morkul.__init__(self, player)
    self.grab_item(Key())

class Morkul3(Morkul):
  pass



def GotoBuilding(character, BuildingClass, limit = 12.0):
  print character.level.of_class(BuildingClass)
  
  yield Goto(character, character.level.of_class(BuildingClass), limit)
  yield Wait(character, 500)
  
def MorkulDiscussion(character):
  from balazar.player   import CURRENT_CHARACTER as hero
  from balazar.building import MorkulTemple, TrueTemple
  
  if isinstance(character, Morkul1) and not character.item_of_class(Key):
    intro = _(u"Bonsssoir tranger. Je cherche une clef dor que j'ai perdu il y a quelques jours.")
    hero.set_spc("know_morkul1_search_key")
  else:
    intro = _(u"Bonsssoir tranger.")
    
  return Discussion(character, u"", intro, choices = lambda : (
    Discussion(character,
               _(u"Je n'ai pas vu ta clef"),
               _(u"""Si tu la trouve, apporte-la moi, je t'en prie."""),
               hero.get_spc("know_morkul1_search_key") and isinstance(character, Morkul1) and not hero.item_of_class(Key),
               ),
    
    Discussion(character,
               _(u"J'ai retrouv ta clef, la voici"),
               _(u"""Merci, merci ! Dsormais, tu seras mon ami ! Je m'appelle Croky."""),
               hero.get_spc("know_morkul1_search_key") and isinstance(character, Morkul1) and hero.item_of_class(Key),
               action = lambda : (hero.set_spc("know_morkul1_search_key", 0), character.change_relation(hero, 1.5), character.grab_item(hero.remove_item_of_class(Key))),
               ),
    
    
    Discussion(character,
               _(u"J'ai retrouv ta hache, la voici"),
               _(u"""Merci, comme convenu en change je te donne la clef dore."""),
               hero.get_spc("know_morkul2_search_axe") and isinstance(character, Morkul2) and hero.item_of_class(Axe),
               action = lambda : (hero.set_spc("know_morkul2_search_axe", 0), character.change_relation(hero, 0.5), character.grab_item(hero.remove_item_of_class(Axe)), hero.grab_item(character.remove_item_of_class(Key))),
               ),
    
    
    Discussion(character,
               _(u"Qui es-tu, crature ?"),
               _(u"Je sssuis un Morkul, un dmon de la nuit, buveur de ssang toujours \
asssoiff... Et toi qui es-tu ? Ton sang peut-il se boire ?"), choices = lambda : (
      Discussion(character,
                 _(u"Je suis Balazar le sorcier, et mon sang a trs mauvais got"),
                 _(u"Jamais entendu parl... mais je gouterais bien cette nouveaut..."),
                 action = lambda : character.change_relation(hero, -0.4),
                 ),
      Discussion(character,
                 _(u"Personne... juste un humain"),
                 _(u"Les humains font une chair mdiocre... comme le reste d'ailleurs")),
      Discussion(character,
                 _(u"Je suis un habitant d'un autre monde qui joue  un jeu vido"),
                 _(u"Ben Voyons...")),
      )),
    
    Discussion(character,
               _(u"Qui es-tu, crature ?"),
               _(u"""Je sssuis Morkar, le gardien du temple des Morkuls. Tous ssseux de ma rassse \
craignent et redoutent mon nom... de mme que les autres, bien sr !"""),
               isinstance(character, Morkar),
               ),
    
    Discussion(character,
               _(u"As-tu vu un temple dans la rgion ?"),
               _(u"C'est du temple des Morkuls que tu veux parler. Aucun temple ne peux \
galer sssa beaut."),
               action = lambda : hero.set_spc("know_morkul_temple", 1),
               choices = lambda : (
    
      Discussion(character,
                 _(u"O est le temple des Morkuls ?"),
                 _(u"Il n'est pas trs loin d'isssi... non, tout prs mme... Veux-tu que \
je t'y conduise ?"),
                 hero.get_spc("know_morkul_temple"), choices = lambda : (
        Discussion(character,
                   _(u"Oui"),
                   _(u"Sssuis-moi, tranger..."), choices = [
          Discussion(character,
                     _(u"C'est parti !"), u"",
                     action = lambda : (hero.set_spc("guided_to_morkul_temple"), character.queue.controler.animate_and_cancel(GotoBuilding(character, MorkulTemple, 7.0))),
                     )
                   ]),
        Discussion(character,
                   _(u"Non, je ne veux pas te dranger"),
                   _(u"Comme tu voudras... mais le temple est sssi difficile  trouver...")),
                 )),
               )),
    
    Discussion(character,
               _(u"As-tu vu un temple dans la rgion ?"),
               _(u"""Je suis le gardien du temple des Morkuls. Si tu ne vois pas le temple, \
c'est que tu es aveugle ou idiot ! Qu'attends-tu pour le prendre en photo ?"""),
               isinstance(character, Morkar),
               lambda : hero.set_spc("know_morkul_temple", 1),
               choices = lambda : (
    
      Discussion(character,
                 _(u"Je le prends en photo tout de suite."), u""),
                
      Discussion(character,
                 _(u"Ce n'est pas le temple que je cherche ! Il y a erreur !"),
                 _(u"Tu parles trop... je vais t'apprendre  te taire."),
                 action = lambda : character.change_relation(hero, -1.5),
                 ),
               )),
    
    
    Discussion(character,
               _(u"N'y a-t-il pas un autre temple dans la rgion ?"),
               _(u"Non non non. Je suis sssertains qu'il n'y en a pas."),
               hero.get_spc("know_morkul_temple_is_bad") and not isinstance(character, Morkul1), choices = lambda : (
               )),
    
    Discussion(character,
               _(u"N'y a-t-il pas un autre temple dans la rgion ?"),
               _(u"Je te le dirais peut-tre sssi tu retrouves ma clef..."),
               hero.get_spc("know_morkul_temple_is_bad") and hero.get_spc("know_morkul1_search_key") and isinstance(character, Morkul1), choices = lambda : (
               )),
    
    Discussion(character,
               _(u"Mais dis moi, entre ami... on se dit o sont les temples secrets ?"),
               _(u"Morkar notre chef ne veut pas que l'on en parle... Sssuis moi discrtement, je vais te conduire au temple et je lverai le brouillard magique qui le dissimule."),
               hero.get_spc("know_morkul_temple_is_bad") and isinstance(character, Morkul1) and hero.get_relation(character) >= 1.5,
               choices = [
      Discussion(character,
                 _(u"C'est parti !"), u"",
                 action = lambda : (hero.set_spc("guided_to_true_temple"), character.level.of_class(TrueTemple).set_activated(1), character.queue.controler.animate_and_cancel(GotoBuilding(character, TrueTemple))),
                 ),
               ]),
    
    Discussion(character,
               _(u"Tu m'as tromp, ce n'est pas le vrai temple ! Je connais les cratures de ton espce !"),
               _(u"Mais pourquoi tant de mchanssst ?"),
               hero.get_spc("know_morkul_temple_is_bad") and hero.get_spc("guided_to_morkul_temple"),
               lambda : character.change_relation(hero, -0.5),
               ),
    
    Discussion(character,
               _(u"As-tu vu une clef dor ?"),
               _(u"Une clef ? Non."),
               hero.get_spc("know_morkul1_search_key") and (not isinstance(character, Morkul1)) and (not hero.item_of_class(Key)),
               ),
    
    Discussion(character,
               _(u"As-tu vu une clef dor ?"),
               _(u"Oui, j'en ai trouv une l'autre jour."),
               hero.get_spc("know_morkul1_search_key") and isinstance(character, Morkul2) and (not hero.item_of_class(Key)), choices = lambda : (
      Discussion(character,
                 _(u"Donne-la moi, je sais  qui elle appartient"),
                 _(u"C'est moi qui l'ai trouv ! Je te la donnerais si en change tu retrouves la hache que j'ai perdu hier en abattant des champignons."),
                 action = lambda : hero.set_spc("know_morkul2_search_axe"),
                 choices = lambda : (
        Discussion(character,
                   _(u"Je vais chercher ta hache"),
                   ),
        Discussion(character,
                   _(u"Je connais une autre faon de rcuprer cette clef... sur ton cadavre !"),
                   action = lambda : character.change_relation(hero, -3.0),
                   ),
        )),
      )),
    
    Discussion(character,
               [_(u"Tu vas mourir, crature des tnbres !"),
                _(u"Prpare-toi  mourir, face de rat !"),
                _(u"Morkul, tss... je vais te montrer de quoi est capable un vrai dmon !"),
                ][hero.get_delta_alignment(character)],
               action = lambda : character.change_relation(hero, -3.0),
               ),
    ))
