# Balazar
# Copyright (C) 2003-2004 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from sets import *
import math
import soya

class PhotoScoreable:
  no_close_up = 1
  def get_photo_score(self, photo, camera):
    return DEFAULT_SCORE

class Wonder(PhotoScoreable):
  pass
  
  
DEFAULT_SCORE = 0.0, 0.0, 0.0

SHAPES_SCORES = {
  "dent"                  : ( 0.0, 10.0, 2.0),
  "champignon@pos1"       : ( 5.0,  3.0, 2.0),
  "champignon@pos2"       : ( 6.0,  3.0, 2.0),
  "champignon@pos3"       : ( 0.0,  7.0, 2.0),
  "fasme@var1"            : ( 8.0,  0.0, 2.0),
  "fasme@var2"            : ( 8.0,  0.0, 2.0),
  "marchee_etalage"       : (15.0,  0.0, 5.0),
  "ferme"                 : (12.0,  0.0, 4.0),
  "tour"                  : (10.0,  5.0, 4.0),
  "marchee"               : (50.0,  0.0, 6.0),
  "scn-tree5"             : (15.0,  0.0, 5.0),
  }

WEATHER_SCORES = {
  "sunny" : ( 0.0,  0.0),
  "rain"  : ( 2.0, 20.0),
  "snow"  : (20.0,  0.0),
  }

def evaluate(user, photo, camera):
  score = 0
  
  distance = camera.back
  
  rc  = camera.get_root()
  p   = soya.Point ()
  v   = soya.Vector()
  dir = soya.Vector()
  content = []
  for item in camera.get_root().recursive():
    if isinstance(item, PhotoScoreable) and item.visible and camera.is_in_frustum(item):
      if 1:
        item_dist = camera.distance_to(item) - 0.1
        dir.set_start_end(camera, item)
        impact = rc.raypick(camera, dir, distance, 1, 1, p, v)
        if impact and not(p.parent.is_inside(item) or (camera.distance_to(p) >= item_dist)):
          p.__init__(item, 0.0, 1.0, 0.0)
          dir.set_start_end(camera, p)
          impact = rc.raypick(camera, dir, distance, 1, 1, p, v)
          if impact and not(p.parent.is_inside(item) or (camera.distance_to(p) >= item_dist)):
            p.__init__(item, -1.0, 0.0, 0.0)
            dir.set_start_end(camera, p)
            impact = rc.raypick(camera, dir, distance, 1, 1, p, v)
            if impact and not(p.parent.is_inside(item) or (camera.distance_to(p) >= item_dist)):
              p.__init__(item, 1.0, 0.0, 0.0)
              dir.set_start_end(camera, p)
              impact = rc.raypick(camera, dir, distance, 1, 1, p, v)
              if impact and not(p.parent.is_inside(item) or (camera.distance_to(p) >= item_dist)):
                p.__init__(item, 0.0, 0.0, -1.0)
                dir.set_start_end(camera, p)
                impact = rc.raypick(camera, dir, distance, 1, 1, p, v)
                if impact and not(p.parent.is_inside(item) or (camera.distance_to(p) >= item_dist)):
                  p.__init__(item, 0.0, 0.0, 1.0)
                  dir.set_start_end(camera, p)
                  impact = rc.raypick(camera, dir, distance, 1, 1, p, v)
                  if impact and not(p.parent.is_inside(item) or (camera.distance_to(p) >= item_dist)):
                    continue
                  
      content.append(item)
      
#       if isinstance(item, soya.World):
#         w = item
#       else:
#         w = soya.World(item.parent)
#         w.add(item)
#       w.atmosphere = soya.Atmosphere()
#       w.atmosphere.ambient = (1.0, 0.0, 0.0, 1.0)
      
  print content
  
  from balazar.base import display_name
  
  scored_content = {}
  clazz_content = Set()
  
  total      = 1.0
  score1     = 0.0
  score2     = 0.0
  nb_wonders = 0
  for item in content:
    if isinstance(item, Wonder): nb_wonders = 1
    
    sco1, sco2, tot = item.get_photo_score(photo, camera)
    dist = item.distance_to(camera)
    if (not item.no_close_up) or (dist > 3.0 * tot):
      dist = 1.0 + (dist // (10.0 + 5.0 * tot))
      sco1    = sco1 * tot / dist
      sco2    = sco2 * tot / dist
      score1 += sco1
      score2 += sco2
      total  +=        tot / dist
      
      dn = display_name(item)
      scored_content[dn] = scored_content.get(dn, 0.0) + sco1 + sco2
      clazz_content.add(item.__class__)
      
      print item, sco1 * tot / dist, sco2 * tot / dist
      
    else:
      print item, 0.0, 0.0
      
  day_night_cycle = user.level.atmosphere
  if day_night_cycle:
    sco1, sco2 = WEATHER_SCORES[day_night_cycle.weather]
    score1 += sco1
    score2 += sco2
    
    sco1, sco2 = day_night_cycle.get_photo_score(photo, camera)
    score1 += sco1
    score2 += sco2
    
  total  = max(total, 10.0)
  score1 = int(10.0 * score1 / total)
  score2 = int(10.0 * score2 / total)
  
  scored_content = [(score, dn) for dn, score in scored_content.items()]
  scored_content.sort()
  scored_content.reverse()
  scored_content = [dn for score, dn in scored_content]
  
  if score1 > score2: photo.score = score1 - score2 / 2.0
  else:               photo.score = score2 - score1 / 2.0
  photo.content       = scored_content
  photo.nb_wonders    = nb_wonders
  photo.clazz_content = list(clazz_content)
  
  print score1, score2
  print scored_content
  print photo.score

