# Balazar
# Copyright (C) 2003 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, popen2, signal, time

import soya, soya.widget as widget
from soya import Point, Vector

import balazar.globdef as globdef, balazar.hero as hero, balazar.controler as controler
from balazar.controler import KeyboardMouseControler
from balazar.sound     import end_music
from balazar.level     import random_level, Idler, put_on_ground
from balazar.camera    import Traveling

CURRENT_CHARACTER = None

def start_single_game():
  global CURRENT_CHARACTER
  
  idler = Idler(soya.World())
  idler.min_frame_duration = 0.025
  
  level     = random_level()
  character = CURRENT_CHARACTER = hero.Balazar(1)
  #import balazar.npc; character = CURRENT_CHARACTER = balazar.npc.Morkul(1)
  character.id = 1
  character.name = "player1"
  # Do this BEFORE set_level, since set_level will send the character
  
  p = soya.Point(level, 0.0, 1000.0, 0.0)
  down = soya.Vector(level, 0.0, -1.0, 0.0)
  while 1:
    r = level.raypick(p, down)
    p.x = p.z = p.x + 2.0
    if r: break
  r = level.raypick(p, down)
  
  import balazar.vehicule
  boat = balazar.vehicule.FlyingBoat(level)
  boat.set_xyz(-50.0, r[0].y + 5.5, -50.0)
  boat.rotate_lateral(180.0 + 45.0)
  
  character.move(soya.Point(boat, 0.0, 1.5, -1.3) % level)
  #character.set_xyz(150.0, 150.0, 150.0)
  put_on_ground(level, None, character)
  
  character.rotate_lateral(180.0 + 45.0)
  
  return build_scene(idler, level, character)


def build_scene(idler, level, character):
  character.set_controler(KeyboardMouseControler(character))
  character.zap()
  
  if level: character.set_level(level)
  
  camera = idler.camera = idler.scene.camera = soya.TravelingCamera(idler.scene)
  if globdef.SOUND: soya.openal4soya.init(camera)
  
  traveling = Traveling(character, Vector(None, 0.0, 0.0, 2.0))
  #traveling.incline_as = None
  camera.add_traveling(traveling)
  camera.back = 100.0
  camera.speed = 0.4
  
  soya.set_root_widget(RootGroup())
  main_view = MainViewGroup(soya.root_widget)
  main_view.add(camera)
  #widget.FPSLabel(main_view)

  import balazar.sound
  balazar.sound.play_music("ocane.ogg")

  import balazar.discussion
  #balazar.discussion.DiscussionWidget(soya.root_widget, character, balazar.discussion.Help())
  balazar.discussion.StartDiscussion(character).activate(1)
  
  character.queue.controler.init()
  
  soya.process_event() # Clear all events (seems that joystick causes events at the beginning)
  
  result = idler.idle()
  # Del these ref in order to clean_mem() to be able to work.
  idler = None
  level = None
  player = None
  clean_mem()

  end_music()

  return result
  
  
def clean_mem():
  import balazar.character as balazar_character
  
  soya.set_root_widget(None)
  
  if globdef.SOUND:
    soya.openal4soya._CAMERA = None
    soya.openal4soya._SOURCES_POS_SPEED *= 0
    
  import gc; gc.collect() # Perform GC after, rather than while, playing !!!



class RootGroup(widget.Group):
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    widget.Group.resize(self, parent_left, parent_top, parent_width, parent_height)
    
class MainViewGroup(widget.Group):
  def __init__(self, master):
    widget.Group.__init__(self, master)
    master.main_view = self
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    widget.Group.resize(self, parent_left, parent_top, parent_width, int(parent_height - controler.TEXT_HEIGHT - 7))
